@@ -25,6 +25,7 @@
 #include <falcon/genhasm.h>
 #include <falcon/gentree.h>
 #include <falcon/gencode.h>
+#include <falcon/signals.h>
 #include <iostream>
 #include <stdio.h>
 
@@ -45,7 +46,10 @@ AppFalcon::AppFalcon():
    m_script_pos( 0 )
 {
    // Install a void ctrl-c handler (let ctrl-c to kill this app)
-   Falcon::Sys::_dummy_ctrl_c_handler();
+   Sys::_dummy_ctrl_c_handler();
+
+   // Block all signals in the main thread.
+   BlockSignals();
 
    // Prepare the Falcon engine to start.
    Engine::Init();
@@ -301,7 +305,7 @@ Stream* AppFalcon::openOutputStream( const String &ext )
       if ( m_options.input != "" && m_options.input != "-" )
       {
          #ifdef WIN32
-            Sys::falconConvertWinFname( m_options.input );
+            Path::winToUri( m_options.input );
          #endif
          URI uri_input( m_options.input );
          uri_input.pathElement().setFilename( uri_input.pathElement().getFile() +
@@ -342,9 +346,11 @@ Module* AppFalcon::loadInput( ModuleLoader &ml )
    if ( m_options.input != "" && m_options.input != "-" )
    {
       #ifdef WIN32
-         Sys::falconConvertWinFname( m_options.input );
+         Path::winToUri( m_options.input );
       #endif
-      mod = ml.loadFile( m_options.input, ModuleLoader::t_none, true );
+      mod = ml.loadFile( m_options.input,
+            m_options.run_only ? ModuleLoader::t_vmmod : ModuleLoader::t_defaultSource,
+            false );
    }
    else
    {
@@ -447,7 +453,7 @@ void AppFalcon::generateTree()
    if ( m_options.input != "" && m_options.input != "-" )
    {
       #ifdef WIN32
-         Sys::falconConvertWinFname( m_options.input );
+         Path::winToUri( m_options.input );
       #endif
       FileStream* fs = new FileStream();
       fs->open( m_options.input );
@@ -610,7 +616,10 @@ void AppFalcon::runModule()
    //===========================================
    // Prepare the virtual machine
    //
-   VMachineWrapper vmachine;
+
+   VMachine* vm = new VMachine( false );
+   VMachineWrapper vmachine(vm);
+
 
    //redirect the VM streams to ours.
    // The machine takes ownership of the streams, so they won't be useable anymore
@@ -619,6 +628,7 @@ void AppFalcon::runModule()
    vmachine->stdIn( m_stdIn );
    vmachine->stdOut( m_stdOut );
    vmachine->stdErr( m_stdErr );
+   vmachine->init();
    // I have given real process streams to the vm
    vmachine->hasProcessStreams( true );
 
@@ -666,6 +676,9 @@ void AppFalcon::runModule()
    vmachine->launchAtLink( true );
    if ( vmachine->link( &runtime ) )
    {
+      // Broadcast OS signals in this VM.
+      //vmachine->becomeSignalTarget();
+
       vmachine->launch();
 
       if ( vmachine->regA().isInteger() )
@@ -715,8 +728,8 @@ void AppFalcon::run()
 
 int main ( int argc, char *argv[] )
 {
-   StdErrStream serr;
    AppFalcon falcon;
+   StdErrStream serr;
 
    try {
       if ( falcon.setup( argc, argv ) )
