@@ -71,15 +71,7 @@ void AutoWString::init_vm_and_format( VMachine *vm, const Item &itm, const Strin
 
    str = &tempStr;
    vm->itemToString( tempStr, &itm, fmt );
-   if( vm->hadError() )
-   {
-      m_pData = m_buffer;
-      // as 255 is not a valid UTF-16 character at the beginning of a string,
-      // we can use it to determine if the string is valid.
-      m_buffer[0] = (wchar_t) 0xFFFFF;
-      return;
-   }
-
+   
    if ( ( m_len = str->toWideString( m_buffer, AutoWString_BUF_SPACE ) ) != String::npos )
    {
       m_pData = m_buffer;
