@@ -17,16 +17,23 @@
 #include <falcon/syntree.h>
 #include <falcon/src_lexer.h>
 #include <falcon/error.h>
-#include <falcon/deferrorhandler.h>
 #include <falcon/stdstreams.h>
 #include <falcon/itemid.h>
 #include <falcon/fassert.h>
+#include <falcon/path.h>
+#include <falcon/intcomp.h>
+#include <falcon/modloader.h>
+#include <falcon/vm.h>
+#include <falcon/stringstream.h>
+#include "core_module/core_module.h"
+
+#include <math.h>
 
 namespace Falcon
 {
 
 AliasMap::AliasMap():
-   Map( &traits::t_stringptr, &traits::t_voidp )
+   Map( &traits::t_stringptr(), &traits::t_voidp() )
 {
 }
 
@@ -35,25 +42,39 @@ AliasMap::AliasMap():
 //===============================================================
 
 Compiler::Compiler( Module *mod, Stream* in ):
-   m_errhand(0),
-   m_module(0),
+   m_constants( &traits::t_string(), &traits::t_voidp() ),
+   m_namespaces( &traits::t_string(), &traits::t_voidp() ),
+   m_root(0),
+   m_errors(0),
+   m_optLevel(0),
+
+   m_lexer(0),
    m_stream( in ),
-   m_constants( &traits::t_string, &traits::t_voidp ),
+
+   m_module(0),
+   m_enumId(0),
+
+   m_staticPrefix(0),
+   m_lambdaCount(0),
+   m_closureContexts(0),
+   m_tempLine(0),
+
    m_strict( false ),
    m_language( "C" ),
    m_modVersion( 0 ),
    m_defContext( false ),
-   m_delayRaise( false ),
+   m_bParsingFtd(false),
+   m_bInteractive( false ),
+
    m_rootError( 0 ),
-   m_lexer(0),
-   m_root(0),
-   m_bParsingFtd(false)
+   m_metacomp( 0 ),
+   m_serviceVM( 0 ),
+   m_serviceLoader( 0 )
 {
-   // Initializing now prevents adding predefined constants to the module.
    init();
 
    m_module = mod;
-   m_module->engineVersion( FALCON_VERSION_NUM );
+   m_module->engineVersion( (FALCON_VERSION_NUM) );
 
    addPredefs();
 
@@ -62,22 +83,31 @@ Compiler::Compiler( Module *mod, Stream* in ):
 }
 
 Compiler::Compiler():
-   m_errhand(0),
-   m_module( 0 ),
+   m_constants( &traits::t_string(), &traits::t_voidp() ),
+   m_namespaces( &traits::t_string(), &traits::t_voidp() ),
+   m_root(0),
+   m_errors(0),
+   m_optLevel(0),
+   m_lexer(0),
    m_stream( 0 ),
-   m_constants( &traits::t_string, &traits::t_voidp ),
+   m_module( 0 ),
+   m_enumId( 0 ),
+   m_staticPrefix(0),
+   m_lambdaCount(0),
+   m_closureContexts(0),
+   m_tempLine(0),
    m_strict( false ),
    m_language( "C" ),
    m_modVersion( 0 ),
    m_defContext( false ),
-   m_delayRaise( false ),
+   m_bParsingFtd(false),
+   m_bInteractive( false ),
    m_rootError( 0 ),
-   m_lexer(0),
-   m_root(0),
-   m_lambdaCount(0),
-   m_bParsingFtd(false)
+   m_metacomp(0),
+   m_serviceVM( 0 ),
+   m_serviceLoader( 0 )
 {
-   reset();
+   addPredefs();
 }
 
 Compiler::~Compiler()
@@ -95,7 +125,7 @@ Compiler::~Compiler()
 
 void Compiler::init()
 {
-   m_lexer = new SrcLexer( this, m_stream );
+   m_lexer = new SrcLexer( this );
    if ( m_bParsingFtd )
       m_lexer->parsingFtd( true );
 
@@ -127,22 +157,13 @@ void Compiler::reset()
       clear();
    }
 
-   // reset contants
-   MapIterator iter = m_constants.begin();
-   while( iter.hasCurrent() ) {
-      delete *(Value **) iter.currentValue();
-      iter.next();
-   }
-   m_constants.clear();
+   m_namespaces.clear();
 
    m_alias.clear();
    m_context.clear();
    m_func_ctx.clear();
    m_contextSet.clear();
    m_loops.clear();
-   m_statementVals.clear();
-
-   addPredefs();
 
    m_errors = 0;
    m_enumId = 0;
@@ -158,8 +179,19 @@ void Compiler::reset()
 
 void Compiler::clear()
 {
+
+   if( m_serviceVM != 0 )
+      m_serviceVM->finalize();
+
+   delete m_serviceLoader;
+   delete m_metacomp;
+   m_serviceVM = 0;
+   m_serviceLoader = 0;
+   m_metacomp = 0;
+
    delete m_root;
    delete m_lexer;
+
    m_root = 0;
    m_lexer= 0;
 
@@ -216,7 +248,14 @@ bool Compiler::compile()
    int64 modver = m_modVersion;
 
    // parse
+   m_lexer->input( m_stream );
    flc_src_parse( this );
+   if ( ! m_stream->good() )
+   {
+      raiseError( new IoError( ErrorParam( e_io_error, __LINE__ )
+         .origin( e_orig_compiler )
+         .sysError( (uint32) m_stream->lastError() ) ) );
+   }
    m_module->language( m_language );
    m_module->version( (uint32) m_modVersion );
 
@@ -241,26 +280,9 @@ bool Compiler::compile()
          m_module->symbolTable().exportUndefined();
       }
 
-      // a correctly compiled module has an entry point starting at 0.
-      if ( ! m_root->statements().empty() )
-      {
-         m_module->entry( 0 );
-      }
       return true;
    }
 
-   if ( m_delayRaise && m_rootError != 0 && m_errhand != 0 )
-   {
-      m_errhand->handleError( m_rootError );
-   }
-
-   // eventually prepare for next compilation
-   if ( m_rootError != 0 )
-   {
-      m_rootError->decref();
-      m_rootError = 0;
-   }
-
    // we had errors.
    return false;
 }
@@ -292,25 +314,24 @@ void Compiler::raiseError( int code, const String &errorp, int line )
    if ( line == 0 )
       line = lexer()->line()-1;
 
-   if ( m_errhand != 0 )
+   SyntaxError *error = new SyntaxError( ErrorParam(code, line)
+         .origin( e_orig_compiler ) );
+   error->extraDescription( errorp );
+   error->module( String(m_module->path()).bufferize() );
+
+   raiseError( error );
+}
+
+void Compiler::raiseError( Error *error )
+{
+   if ( m_rootError == 0 )
+   {
+      m_rootError = error;
+   }
+   else
    {
-      SyntaxError *error = new SyntaxError( ErrorParam(code, line).origin( e_orig_compiler ));
-      error->extraDescription( errorp );
       error->module( m_module->path() );
-
-      if ( m_delayRaise )
-      {
-         if ( m_rootError == 0 )
-         {
-            m_rootError = new SyntaxError( ErrorParam( e_syntax ).origin( e_orig_compiler ) );
-         }
-         m_rootError->appendSubError( error );
-      }
-      else
-      {
-         m_errhand->handleError( error );
-      }
-
+      m_rootError->appendSubError( error );
       error->decref();
    }
 
@@ -358,12 +379,18 @@ void Compiler::defineVal( Value *val )
    // raise error for read-only expressions
    if ( val->isExpr() )
    {
+      // byte accessors cannot receive values
       if ( val->asExpr()->type() == Expression::t_array_byte_access )
       {
          raiseError( e_byte_access, lexer()->previousLine() );
          // but proceed
       }
-      else {
+      // assignments to accessors and  function returns doesn't define anything.
+      else if ( ! (val->asExpr()->type() == Expression::t_obj_access ||
+                   val->asExpr()->type() == Expression::t_array_access ||
+                   val->asExpr()->type() == Expression::t_funcall )
+              )
+      {
          Expression *expr = val->asExpr();
          defineVal( expr->first() );
          if( expr->second() != 0 )
@@ -400,9 +427,12 @@ void Compiler::defineVal( Value *val )
       if ( staticPrefix() == 0 )
       {
          Symbol *sym;
-         if ( getFunction() != 0 ) {
+         // are we in a function that misses that symbol?
+         if ( getFunction() != 0 )
+         {
+            // we're creating the local symbol
             // addlocal must also define the symbol
-            sym = addLocalSymbol( val->asSymdef(), false );
+            sym = addLocalSymbol( *val->asSymdef(), false );
          }
          else {
             // globals symbols that have been added as undefined must stay so.
@@ -415,7 +445,7 @@ void Compiler::defineVal( Value *val )
                   raiseError( e_undef_sym, "", lexer()->previousLine() );
                }
 
-               sym = addGlobalSymbol( val->asSymdef() );
+               sym = addGlobalSymbol( *val->asSymdef() );
                sym->declaredAt( lexer()->previousLine() );
                sym->setGlobal();
             }
@@ -424,7 +454,7 @@ void Compiler::defineVal( Value *val )
          val->setSymbol( sym );
       }
       else {
-         String *symname = m_module->addString( *staticPrefix() + "#" + *val->asSymdef() );
+         String symname = *staticPrefix() + "#" + *val->asSymdef();
          Symbol *gsym = addGlobalSymbol( symname );
          AliasMap &map = *(AliasMap*)m_alias.back();
          map.insert( val->asSymdef(), gsym );
@@ -443,7 +473,7 @@ void Compiler::defineVal( Value *val )
 }
 
 
-Symbol *Compiler::addLocalSymbol( const String *symname, bool parameter )
+Symbol *Compiler::addLocalSymbol( const String &symname, bool parameter )
 {
    // fallback to add global if not in a local table
    FuncDef *func = getFunction();
@@ -451,25 +481,38 @@ Symbol *Compiler::addLocalSymbol( const String *symname, bool parameter )
       return addGlobalSymbol( symname );
 
    SymbolTable &table = func->symtab();
-   Symbol *sym = table.findByName( *symname );
+   Symbol *sym = table.findByName( symname );
    if( sym == 0 )
    {
-      if ( m_strict && ! m_defContext )
-      {
-         raiseError( e_undef_sym, "", lexer()->previousLine() );
-      }
-
       // now we can add the symbol. As we have the string from
       // the module already, we keep it.
       sym = new Symbol( m_module, symname );
       m_module->addSymbol( sym );
       sym->declaredAt( lexer()->previousLine() );
+
+      // this flag marks closure forward definitions
+      bool taken = false;
       if ( parameter ) {
          sym = func->addParameter( sym );
       }
       else
       {
-         sym = func->addLocal( sym );
+         // If we're in a closure, we may wish to add
+         // a local undefined that will be filled at closure ending
+         if ( m_closureContexts && searchLocalSymbol( symname, true ) != 0 )
+         {
+            taken = true;
+            sym = func->addUndefined( sym );
+         }
+         else
+            sym = func->addLocal( sym );
+      }
+
+      // in strict mode raise an error if we're not in def, but not if
+      // we taken the symbol from some parent.
+      if ( !taken && m_strict && !m_defContext )
+      {
+         raiseError( e_undef_sym, "", lexer()->previousLine() );
       }
    }
    return sym;
@@ -485,21 +528,34 @@ bool Compiler::checkLocalUndefined()
       if ( val->isSymdef() )
       {
          Symbol *sym = 0;
+
          if ( m_closureContexts > 0 )
          {
-            fassert( m_functions.end() );
-            if ( m_functions.begin() != m_functions.end() )
-            {
-               const FuncDef *fd_parent = reinterpret_cast<const FuncDef *>
-                  ( m_functions.end()->prev()->data() );
+            // still undefined after some loop?
+            // in case of undef == x[undef.len()] we have double
+            // un-definition of undef on the same line.
+            fassert( m_functions.end() != 0 );
+            ListElement* le = m_functions.end();
+            const FuncDef *fd_current = reinterpret_cast<const FuncDef *>( le->data() );
 
-               if( fd_parent != 0 )
+            // try to find it locally -- may have been defined in a previous loop
+            sym = fd_current->symtab().findByName( *val->asSymdef() );
+            if  ( sym == 0 )
+            {
+               le = le->prev();
+               while( le != 0 )
                {
+                  const FuncDef *fd_parent = reinterpret_cast<const FuncDef *>( le->data() );
+
                   if( fd_parent->symtab().findByName( *val->asSymdef() ) != 0 )
                   {
-                     sym = getFunction()->addUndefined(
-                        m_module->addSymbol(*val->asSymdef()) );
+                     sym = m_module->addSymbol(*val->asSymdef());
+                     sym->declaredAt( lexer()->previousLine() );
+                     getFunction()->addUndefined( sym );
+                     break;
                   }
+
+                  le = le->prev();
                }
             }
          }
@@ -507,33 +563,50 @@ bool Compiler::checkLocalUndefined()
          // still nothing?
          if (sym == 0)
          {
-            sym = addGlobalSymbol( val->asSymdef() );
+            // --- import the symbol
+            sym = addGlobalSymbol( *val->asSymdef() );
          }
+
          val->setSymbol( sym );
       }
+
       l->popFront();
    }
    return true;
 }
 
 
-Symbol *Compiler::searchLocalSymbol( const String *symname )
+Symbol *Compiler::searchLocalSymbol( const String &symname, bool recurse )
 {
    if( m_functions.empty() )
       return searchGlobalSymbol( symname );
 
    // first search the local symbol aliases
    AliasMap *map = (AliasMap *) m_alias.back();
-   Symbol **sympp = (Symbol **) map->find( symname );
+   Symbol **sympp = (Symbol **) map->find( &symname );
    if ( sympp != 0 )
       return *sympp;
 
    // then try in the local symtab or just return 0.
-   FuncDef *fd = (FuncDef *) m_functions.back();
-   return fd->symtab().findByName( *symname );
+   ListElement* lastFunc = m_functions.end();
+   while( lastFunc != 0 )
+   {
+      FuncDef *fd = (FuncDef *)lastFunc->data();
+      Symbol *found;
+      if( (found = fd->symtab().findByName( symname )) )
+         return found;
+
+      if ( ! recurse )
+         return 0;
+
+      lastFunc = lastFunc->prev();
+   }
+
+   // definitely not found.
+   return 0;
 }
 
-Symbol *Compiler::searchOuterSymbol( const String *symname )
+Symbol *Compiler::searchOuterSymbol( const String &symname )
 {
    ListElement *aliasIter = m_alias.end();
    ListElement *funcIter = m_functions.end();
@@ -543,13 +616,13 @@ Symbol *Compiler::searchOuterSymbol( const String *symname )
       AliasMap *map = (AliasMap *) aliasIter->data();
 
       // first search the local symbol aliases
-      Symbol **sympp = (Symbol **) map->find( symname );
+      Symbol **sympp = (Symbol **) map->find( &symname );
       if ( sympp != 0 )
          return *sympp;
 
       // then try in the local symtab or just return 0.
       FuncDef *fd = (FuncDef *) funcIter->data();
-      Symbol *sym = fd->symtab().findByName( *symname );
+      Symbol *sym = fd->symtab().findByName( symname );
       if ( sym != 0 )
          return sym;
 
@@ -561,27 +634,57 @@ Symbol *Compiler::searchOuterSymbol( const String *symname )
 }
 
 
-Symbol *Compiler::searchGlobalSymbol( const String *symname )
+Symbol *Compiler::searchGlobalSymbol( const String &symname )
 {
-   return module()->findGlobalSymbol( *symname );
+   return module()->findGlobalSymbol( symname );
 }
 
 
-Symbol *Compiler::addGlobalSymbol( const String *symname )
+Symbol *Compiler::addGlobalSymbol( const String &symname )
 {
    // is the symbol already defined?
-   Symbol *sym = m_module->findGlobalSymbol( *symname );
+   Symbol *sym = m_module->findGlobalSymbol( symname );
+   bool imported = false;
    if( sym == 0 )
    {
+      // check if it is authorized.
+      // Unauthorized symbols are namespaced symbol not declared in import all clauses
+      uint32 dotpos;
+      if( (dotpos = symname.rfind ( "." ) ) != String::npos )
+      {
+         // Namespaced symbol
+         String nspace = symname.subString( 0, dotpos );
+         // change self into our name
+
+         void **mode = (void **) m_namespaces.find( &nspace );
+
+         // if it's not a namespace, then it's an hard-built symbol as i.e. class._init
+         if ( mode != 0 )
+         {
+            // we wouldn't have a namespaced symbol if the lexer didn't find it was already in
+            if( *mode == 0 )
+            {
+               // if we were authorized, the symbol would have been created by
+               // the clauses itself.
+               raiseError( e_undef_sym, symname );
+               // but add it anyhow.
+            }
+
+            // namespaced symbols are always imported
+            imported = true;
+         }
+      }
+
       sym = new Symbol( m_module, symname );
       m_module->addGlobalSymbol( sym );
-      sym->declaredAt( lexer()->line() );
+      sym->declaredAt( lexer()->previousLine() );
+      sym->imported( imported );
    }
    return sym;
 }
 
 
-Symbol *Compiler::addGlobalVar( const String *symname, VarDef *value )
+Symbol *Compiler::addGlobalVar( const String &symname, VarDef *value )
 {
    Symbol *sym = addGlobalSymbol( symname );
    sym->declaredAt( lexer()->previousLine() );
@@ -589,28 +692,7 @@ Symbol *Compiler::addGlobalVar( const String *symname, VarDef *value )
    return sym;
 }
 
-
-Symbol *Compiler::addAttribute( const String *symname )
-{
-   // find the global symbol for this.
-   Symbol *sym = searchGlobalSymbol( symname );
-
-   // Not defined?
-   if( sym == 0 ) {
-      sym = addGlobalSymbol( symname );
-      sym->declaredAt( lexer()->previousLine() );
-   }
-   else {
-      raiseError( e_already_def,  sym->name() );
-   }
-   // but change it in an attribute anyhow
-   sym->setAttribute();
-
-   return sym;
-}
-
-
-Symbol *Compiler::globalize( const String *symname )
+Symbol *Compiler::globalize( const String &symname )
 {
    if ( ! isLocalContext() ) {
       // an error should be raised elsewhere.
@@ -619,25 +701,25 @@ Symbol *Compiler::globalize( const String *symname )
 
    // already alaised? raise an error
    AliasMap &map = *(AliasMap *) m_alias.back();
-   Symbol **ptr = (Symbol **) map.find( symname );
+   Symbol **ptr = (Symbol **) map.find( &symname );
    if( ptr != 0 )
    {
-      raiseError( e_global_again, *symname );
+      raiseError( e_global_again, symname );
       return *ptr;
    }
 
    // search for the global symbol that will be aliased
-   Symbol *global = m_module->findGlobalSymbol( *symname );
+   Symbol *global = m_module->findGlobalSymbol( symname );
    if ( global == 0 )
    {
-      global = m_module->addGlobal( *symname, false );
+      global = m_module->addGlobal( symname, false );
       global->declaredAt( lexer()->line() );
       // it's defined in the module, the reference will be overwritten with
       // defineVal() -- else it will be searched outside the module.
       // (eventually causing a link error if not found).
    }
 
-   map.insert( symname, global );
+   map.insert( &symname, global );
    return global;
 }
 
@@ -655,7 +737,7 @@ StmtFunction *Compiler::buildCtorFor( StmtClass *cls )
    String cname = sym->name() + "._init";
 
    // creates an empty symbol
-   Symbol *funcsym = addGlobalSymbol(  addString( cname ) );
+   Symbol *funcsym = addGlobalSymbol( cname );
    //def->addProperty( addString( "_init" ) , new VarDef( funcsym ) );
 
    // creates the syntree entry for the symbol; we are using the same line as the class.
@@ -663,13 +745,13 @@ StmtFunction *Compiler::buildCtorFor( StmtClass *cls )
    addFunction( stmt_ctor );
 
    // fills the symbol to be a valid constructor
-   FuncDef *fdef = new FuncDef( 0 );
+   FuncDef *fdef = new FuncDef( 0, 0 );
    funcsym->setFunction( fdef );
    def->constructor( funcsym );
 
    // now we must copy the parameter of the class in the parameters of the constructor.
    MapIterator iter = def->symtab().map().begin();
-   GenericVector params( &traits::t_voidp );
+   GenericVector params( &traits::t_voidp() );
 
    while( iter.hasCurrent() )
    {
@@ -695,12 +777,12 @@ void Compiler::addPredefs()
 
    addIntConstant( "NilType", FLC_ITEM_NIL );
    addIntConstant( "BooleanType", FLC_ITEM_BOOL );
-   addIntConstant( "IntegerType", FLC_ITEM_INT );
+   //addIntConstant( "IntegerType", FLC_ITEM_INT );
    addIntConstant( "NumericType", FLC_ITEM_NUM );
    addIntConstant( "RangeType", FLC_ITEM_RANGE );
-   addIntConstant( "AttributeType", FLC_ITEM_ATTRIBUTE );
    addIntConstant( "FunctionType", FLC_ITEM_FUNC );
    addIntConstant( "StringType", FLC_ITEM_STRING );
+   addIntConstant( "LBindType", FLC_ITEM_LBIND );
    addIntConstant( "MemBufType", FLC_ITEM_MEMBUF );
    addIntConstant( "ArrayType", FLC_ITEM_ARRAY );
    addIntConstant( "DictionaryType", FLC_ITEM_DICT );
@@ -708,6 +790,22 @@ void Compiler::addPredefs()
    addIntConstant( "ClassType", FLC_ITEM_CLASS );
    addIntConstant( "MethodType", FLC_ITEM_METHOD );
    addIntConstant( "ClassMethodType", FLC_ITEM_CLSMETHOD );
+   addIntConstant( "UnboundType", FLC_ITEM_UNB );
+
+}
+
+
+void Compiler::addAttribute( const String &name, Value *val, uint32 line )
+{
+   String n = name;
+   FuncDef* fd = getFunction();
+   if( fd == 0 )
+   {
+      m_module->addAttribute( name, val->genVarDef() );
+   }
+   else {
+      fd->addAttrib(name, val->genVarDef() );
+   }
 }
 
 void Compiler::addIntConstant( const String &name, int64 value, uint32 line )
@@ -722,7 +820,14 @@ void Compiler::addNilConstant( const String &name, uint32 line )
 
 void Compiler::addStringConstant( const String &name, const String &value, uint32 line )
 {
-   addConstant( name, new Value( m_module->addString( value ) ), line );
+   if ( m_module != 0 )
+      addConstant( name, new Value( m_module->addString( value ) ), line );
+   else {
+      // we'll leak, but oh, well...
+      //TODO: fix the leak
+      addConstant( name, new Value( new String( value ) ), line );
+   }
+
 }
 
 void Compiler::addNumConstant( const String &name, numeric value, uint32 line )
@@ -893,7 +998,7 @@ Value *Compiler::closeClosure()
    if( fd->undefined() > 0 )
    {
       // we have to find all the local variables that exist in the upper context and
-      // transform them in parameters.
+      // transform them in the first part of the local array.
       SymbolTable &funcTable = fd->symtab();
       ArrayDecl *closureDecl = new ArrayDecl;
 
@@ -907,45 +1012,66 @@ Value *Compiler::closeClosure()
          Symbol *sym = *(Symbol **) iter.currentValue();
          if ( sym->isLocalUndef() )
          {
-            // ok, this must become a parameter...
-            sym->setParam();
+            // ok, this must become a local symbol...
+            sym->setLocal();
             sym->itemId( moved );
             moved ++;
 
-            Symbol *parentSym;
-            if ( (parentSym = searchLocalSymbol( &sym->name() )) != 0 )
+            // and now let's find the father's having this local undefined.
+            fassert( m_functions.end() != 0 );
+            ListElement* le = m_functions.end();
+            Symbol *parentSym = 0;
+
+            while( le != 0 )
             {
-               //... and the parent symbol must be stored in parametric array...
-               closureDecl->pushBack( new Value( parentSym ) );
+               FuncDef *fd_parent = ( FuncDef *) le->data();
+
+               // when we find it...
+               if( ( parentSym = fd_parent->symtab().findByName( sym->name() )) != 0 )
+               {
+                  // we must now force the declaration of local undefined for this
+                  // symbol up to this closure, so that it sent down the stack on runtime.
+                  le = le->next();
+                  while( le != 0 )
+                  {
+                     fd_parent = (FuncDef *) le->data();
+                     parentSym = fd_parent->addUndefined( m_module->addSymbol( sym->name() ) );
+                     le = le->next();
+                  }
+
+                  //... and the parent symbol must be stored in parametric array...
+                  closureDecl->pushBack( new Value( parentSym ) );
+                  break;
+               }
+
+               le = le->prev();
             }
-            else {
+
+            if ( parentSym == 0 )
+            {
                // closures can't have undefs.
                raiseError( e_undef_sym, "", sym->declaredAt() );
             }
          }
-         else if ( sym->isParam() )
+         else if ( sym->isLocal() )
          {
-            // push forward all parameters
+            // push forward all the locals
             sym->itemId( fd->undefined() + sym->itemId() );
          }
 
          iter.next();
       }
 
-      // no more undefs -- now they are params
-      fd->params( fd->params() + fd->undefined() );
+      // no more undefs -- now they are locals
+      fd->locals( fd->locals() + fd->undefined() );
       fd->undefined( 0 );
 
-      // ... put it in front of our array and return it.
-      closureDecl->pushFront( lambda_call );
-      return new Value( closureDecl );
-   }
-   else
-   {
-      // just create create a lambda call for this function
-      return lambda_call;
+      // ... put as third element of the lambda call.
+      lambda_call->asExpr()->second( new Value( closureDecl ) );
    }
 
+   // just create create a lambda call for this function
+   return lambda_call;
 }
 
 
@@ -960,7 +1086,7 @@ void Compiler::addEnumerator( const String &str, Value *val )
    }
    else
    {
-      VarDef *vd = m_module->addClassProperty( stmt->symbol(), str );
+      VarDef *vd = &m_module->addClassProperty( stmt->symbol(), str );
       switch( val->type() )
       {
          case Value::t_nil :
@@ -984,6 +1110,9 @@ void Compiler::addEnumerator( const String &str, Value *val )
          case Value::t_imm_bool:
             vd->setBool( val->asBool() );
             break;
+
+         default:
+            break;
       }
    }
 }
@@ -995,6 +1124,251 @@ void Compiler::addEnumerator( const String &str )
    addEnumerator( str, &dummy );
 }
 
+
+bool Compiler::isNamespace( const String &symName )
+{
+   return m_namespaces.find( &symName ) != 0;
+}
+
+
+void Compiler::addNamespace( const String &nspace, const String &alias, bool full, bool isFilename )
+{
+   // Purge special namespace names
+   String nselfed;
+
+   if ( alias.size() == 0 )
+   {
+      // if the namespace starts with self, add also the namespace
+      // with the same name of the module
+      if( isFilename )
+      {
+         // we got to convert "/" into "."
+         Path fconv( nspace );
+         nselfed = fconv.getLocation() + "." + fconv.getFile();
+         uint32 pos = 0;
+
+         if ( nselfed.getCharAt(0) == '/' )
+            nselfed = nselfed.subString( 1 );
+
+         while( (pos = nselfed.find( "/", pos ) ) != String::npos ) {
+            nselfed.setCharAt( pos, '.' );
+         }
+      }
+      else if ( nspace.getCharAt(0) == '.' ) {
+         nselfed = nspace.subString( 1 );
+      }
+      else if ( nspace.find( "self." ) == 0 ) {
+         nselfed = nspace.subString(5);
+      }
+      else {
+         nselfed = nspace;
+      }
+   }
+   else {
+         nselfed = alias;
+   }
+
+   // Already added?
+   void **res = (void **) m_namespaces.find( &nselfed );
+
+   if ( res == 0 )
+   {
+      // yes? -- add it
+      m_namespaces.insert( &nselfed, full ? (void *)1 : (void *) 0 );
+
+      // we have to insert in the namespaces all the sub-namespaces.
+      uint32 dotpos = nselfed.find( "." );
+      while( dotpos != String::npos )
+      {
+         String subSpace = nselfed.subString( 0, dotpos );
+
+         // don't overwrite in case we already imported something from it.
+         void *oldNs = m_namespaces.find( &subSpace );
+         if ( oldNs == 0 )
+         {
+            // we set a default of 0 for them as we normally import nothing.
+            m_namespaces.insert( &subSpace, 0 );
+         }
+
+         dotpos = nselfed.find( ".", dotpos+1 );
+      }
+
+      m_module->addDepend( nselfed, nspace, true, isFilename );
+   }
+   // no?
+   else {
+      // -- eventually change to load all.
+      if ( *res == 0 && full )
+         *res = (void *) 1;
+
+      // -- and check if this is a mis-redefinition
+      // raise an error if you try to alias a different module with an alredy existing namespace
+      ModuleDepData* mdPrev = m_module->dependencies().findModule( nselfed );
+      if ( mdPrev != 0 && mdPrev->moduleName() != nspace )
+      {
+         raiseError( e_ns_clash, nselfed, lexer()->previousLine() );
+      }
+   }
+}
+
+
+Symbol *Compiler::importAlias( const String &symName, const String &fromMod, const String &alias, bool filename )
+{
+   // add the dependency
+   m_module->addDepend( fromMod, true, filename );
+
+   // add the alias
+   Falcon::Symbol *sym = new Symbol( m_module, alias );
+   m_module->addGlobalSymbol( sym );
+   sym->declaredAt( lexer()->previousLine() );
+
+   // sets the alias
+   sym->setImportAlias( symName, fromMod, filename );
+
+   return sym;
+}
+
+
+void Compiler::importSymbols( List *lst, const String& pre, const String& alias, bool isFilename )
+{
+   String prefix = pre;
+
+   // add the namespace if not previously known
+   if ( prefix.size() != 0 )
+   {
+      if( alias.size() != 0 )
+      {
+         addNamespace( prefix, alias, false, isFilename );
+         prefix = alias;
+      }
+      else {
+         addNamespace( prefix, "", false, isFilename );
+      }
+   }
+
+   String fprefix;
+   if( isFilename && alias.size() == 0 && prefix.size() != 0 )
+   {
+      // we got to convert "/" into "."
+      Path fconv( prefix );
+      fprefix = fconv.getLocation() + "." + fconv.getFile();
+
+      uint32 pos = 0;
+      if ( fprefix.getCharAt(0) == '/' )
+         fprefix = fprefix.subString(1);
+
+      while( (pos = fprefix.find( "/", pos ) ) != String::npos ) {
+         fprefix.setCharAt( pos, '.' );
+      }
+
+   }
+
+   Falcon::ListElement *li = lst->begin();
+
+   while( li != 0 ) {
+      Falcon::String& symName = *(String *) li->data();
+
+      if( prefix.size() != 0 )
+      {
+         if( isFilename && alias.size() == 0 ) {
+            symName = fprefix + "." + symName;
+         }
+         else if ( prefix.getCharAt(0) == '.' ) {
+            symName = prefix.subString( 1 ) + "." + symName;
+         }
+         else if ( prefix.find( "self." ) == 0 ) {
+            symName = prefix.subString(5) + "." + symName;
+         }
+         else
+            symName = prefix + "." + symName;
+      }
+
+
+      Falcon::Symbol *sym = new Symbol( m_module, symName );
+      m_module->addGlobalSymbol( sym );
+      sym->declaredAt( lexer()->previousLine() );
+      sym->imported(true);
+
+      delete &symName;
+
+      li = li->next();
+   }
+   delete lst;
+}
+
+void Compiler::metaCompile( const String &data, int startline )
+{
+   String ioEnc, srcEnc;
+   Engine::getEncodings( srcEnc, ioEnc );
+
+   // evantually turn on the meta-compiler
+   if ( m_metacomp == 0 )
+   {
+      if ( m_serviceVM == 0 )
+      {
+         m_serviceVM = new VMachine;
+         m_serviceVM->appSearchPath( searchPath() );
+         Module*cm = core_module_init();
+         m_serviceVM->link( cm );
+         cm->decref();
+      }
+
+      if ( m_serviceLoader == 0 )
+      {
+         m_serviceLoader = new ModuleLoader( searchPath() );
+         m_serviceLoader->sourceEncoding( srcEnc );
+      }
+
+      m_metacomp = new InteractiveCompiler( m_serviceLoader, m_serviceVM );
+      m_metacomp->searchPath( searchPath() );
+
+      // not incremental...
+      m_metacomp->lexer()->incremental( false );
+      // do not confuse our user...
+      m_metacomp->module()->path( module()->path() );
+      m_metacomp->module()->name( "[meta] " + module()->name() );
+
+      // transfer the constants.
+      MapIterator iter = m_constants.begin();
+      while( iter.hasCurrent() ) {
+         if ( m_metacomp->m_constants.find( iter.currentKey() ) == 0 )
+         {
+            m_metacomp->addConstant(
+               *(String *)iter.currentKey(),
+               new Value( **(Value **) iter.currentValue()) );
+         }
+         iter.next();
+      }
+
+   }
+
+   StringStream* output = new StringStream;
+   // set the same encoding as the source that we're parsing.
+   m_serviceVM->stdOut( TranscoderFactory( srcEnc, output, true ) );
+
+
+   // set current line in lexer of the meta compiler
+   m_metacomp->tempLine( startline );
+   try
+   {
+      m_metacomp->compileAll( data );
+
+      // something has been written
+      if ( output->length() != 0 )
+      {
+         // pass it to the lexer
+         output->seekBegin(0);
+         StringStream *transferred = new StringStream();
+         transferred->transfer( *output );
+         m_lexer->appendStream( transferred );
+      }
+   }
+   catch( Error *e )
+   {
+      raiseError( e );
+   }
+}
+
 }
 
 /* end of compiler.cpp */
