@@ -18,6 +18,7 @@
 
 #include <falcon/engine.h>
 #include <falcon/filestat.h>
+#include <falcon/complex.h>
 
 namespace Falcon {
 
@@ -35,9 +36,11 @@ FALCON_FUNC GarbagePointer_ptr( VMachine *vm );
 
 FALCON_FUNC core_argv( VMachine *vm );
 FALCON_FUNC core_argd( VMachine *vm );
+FALCON_FUNC core_passvp( VMachine *vm );
 
 // Methodic functions
 FALCON_FUNC  mth_ToString ( ::Falcon::VMachine *vm );
+FALCON_FUNC  mth_bound( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_len ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_typeId ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_compare ( ::Falcon::VMachine *vm );
@@ -56,6 +59,7 @@ FALCON_FUNC  mth_properties( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  Function_name ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Function_caller ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Function_trace ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Function_attributes( ::Falcon::VMachine *vm );
 
 // Iterator class
@@ -95,6 +99,7 @@ FALCON_FUNC  ParseError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC  CloneError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC  IntrruptedError_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC  MessageError_init ( ::Falcon::VMachine *vm );
+FALCON_FUNC  TableError_init ( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  val_int ( ::Falcon::VMachine *vm );
 FALCON_FUNC  val_numeric ( ::Falcon::VMachine *vm );
@@ -144,6 +149,9 @@ FALCON_FUNC  VMSlot_retract( ::Falcon::VMachine *vm );
 FALCON_FUNC  VMSlot_getAssert( ::Falcon::VMachine *vm );
 FALCON_FUNC  VMSlot_first( ::Falcon::VMachine *vm );
 FALCON_FUNC  VMSlot_last( ::Falcon::VMachine *vm );
+FALCON_FUNC  VMSlot_send( ::Falcon::VMachine *vm );
+FALCON_FUNC  VMSlot_register( ::Falcon::VMachine *vm );
+FALCON_FUNC  VMSlot_getEvent( ::Falcon::VMachine *vm );
 
 
 FALCON_FUNC  core_exit ( ::Falcon::VMachine *vm );
@@ -244,6 +252,7 @@ FALCON_FUNC  input ( ::Falcon::VMachine *vm );
 FALCON_FUNC  falcon_getenv( ::Falcon::VMachine *vm );
 FALCON_FUNC  falcon_setenv( ::Falcon::VMachine *vm );
 FALCON_FUNC  falcon_unsetenv( ::Falcon::VMachine *vm );
+FALCON_FUNC  falcon_getEnviron( ::Falcon::VMachine *vm );
 FALCON_FUNC  InputStream_creator ( ::Falcon::VMachine *vm );
 FALCON_FUNC  OutputStream_creator ( ::Falcon::VMachine *vm );
 FALCON_FUNC  IOStream_creator ( ::Falcon::VMachine *vm );
@@ -312,6 +321,11 @@ FALCON_FUNC  mth_strFrontTrim ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_strBackTrim ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_strReplace ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_strReplicate ( ::Falcon::VMachine *vm );
+FALCON_FUNC  mth_strEsq ( ::Falcon::VMachine *vm );
+FALCON_FUNC  mth_strUnesq ( ::Falcon::VMachine *vm );
+FALCON_FUNC  mth_strEscape ( ::Falcon::VMachine *vm );
+FALCON_FUNC  mth_strUnescape ( ::Falcon::VMachine *vm );
+
 FALCON_FUNC  strBuffer ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_strUpper ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_strLower ( ::Falcon::VMachine *vm );
@@ -325,6 +339,7 @@ FALCON_FUNC  String_join ( ::Falcon::VMachine *vm );
 FALCON_FUNC  String_ptr( VMachine *vm );
 FALCON_FUNC  String_charSize( VMachine *vm );
 
+
 FALCON_FUNC  mth_arrayIns ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_arrayDel ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_arrayDelAll ( ::Falcon::VMachine *vm );
@@ -338,10 +353,13 @@ FALCON_FUNC  mth_arrayMerge( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_arrayHead ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_arrayTail ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_arrayFill ( ::Falcon::VMachine *vm );
+FALCON_FUNC  mth_arrayNM( ::Falcon::VMachine *vm );
 FALCON_FUNC  arrayBuffer ( ::Falcon::VMachine *vm );
 FALCON_FUNC  mth_arrayCompact ( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  Array_comp ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Array_mcomp ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Array_mfcomp ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Array_front ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Array_back ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Array_table ( ::Falcon::VMachine *vm );
@@ -366,6 +384,8 @@ FALCON_FUNC  mth_dictFill ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Dictionary_first( ::Falcon::VMachine *vm );
 FALCON_FUNC  Dictionary_last( ::Falcon::VMachine *vm );
 FALCON_FUNC  Dictionary_comp ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Dictionary_mcomp ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Dictionary_mfcomp ( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  fileType( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileNameSplit ( ::Falcon::VMachine *vm );
@@ -385,6 +405,8 @@ FALCON_FUNC  fileMove ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileRemove ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileName ( ::Falcon::VMachine *vm );
 FALCON_FUNC  filePath ( ::Falcon::VMachine *vm );
+FALCON_FUNC  fileExt ( ::Falcon::VMachine *vm );
+FALCON_FUNC  fileUnit ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileChmod ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileChown ( ::Falcon::VMachine *vm );
 FALCON_FUNC  fileChgroup ( ::Falcon::VMachine *vm );
@@ -453,6 +475,8 @@ extern reflectionFuncDecl TimeStamp_timezone_rfrom;
 extern reflectionFuncDecl TimeStamp_timezone_rto;
 
 FALCON_FUNC  Sequence_comp ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Sequence_mcomp ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Sequence_mfcomp ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Sequence_front ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Sequence_back ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Sequence_first ( ::Falcon::VMachine *vm );
@@ -462,6 +486,12 @@ FALCON_FUNC  Sequence_clear ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Sequence_append ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Sequence_prepend ( ::Falcon::VMachine *vm );
 
+FALCON_FUNC Continuation_init ( ::Falcon::VMachine *vm );
+FALCON_FUNC Continuation_call ( ::Falcon::VMachine *vm );
+FALCON_FUNC Continuation_reset ( ::Falcon::VMachine *vm );
+FALCON_FUNC Continuation_complete ( ::Falcon::VMachine *vm );
+FALCON_FUNC Continuation__suspend ( ::Falcon::VMachine *vm );
+
 FALCON_FUNC  List_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC  List_push ( ::Falcon::VMachine *vm );
 FALCON_FUNC  List_pop ( ::Falcon::VMachine *vm );
@@ -491,10 +521,16 @@ reflectionFuncDecl FileStats_atime_rfrom;
 
 FALCON_FUNC  fal_include( ::Falcon::VMachine *vm );
 FALCON_FUNC  Object_attributes( ::Falcon::VMachine *vm );
+FALCON_FUNC  Object_setState( ::Falcon::VMachine *vm );
+FALCON_FUNC  Object_getState( ::Falcon::VMachine *vm );
+FALCON_FUNC  Object_apply( ::Falcon::VMachine *vm );
+FALCON_FUNC  Object_retrieve( ::Falcon::VMachine *vm );
 FALCON_FUNC  Method_attributes( ::Falcon::VMachine *vm );
 FALCON_FUNC  Class_attributes( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  Object_comp( ::Falcon::VMachine *vm );
+FALCON_FUNC  Object_mcomp( ::Falcon::VMachine *vm );
+FALCON_FUNC  Object_mfcomp( ::Falcon::VMachine *vm );
 
 /** Path class */
 FALCON_FUNC Path_init ( ::Falcon::VMachine *vm );
@@ -502,15 +538,21 @@ reflectionFuncDecl Path_path_rfrom;
 reflectionFuncDecl Path_filename_rfrom;
 reflectionFuncDecl Path_unit_rfrom;
 reflectionFuncDecl Path_location_rfrom;
+reflectionFuncDecl Path_fullloc_rfrom;
 reflectionFuncDecl Path_file_rfrom;
 reflectionFuncDecl Path_extension_rfrom;
+reflectionFuncDecl Path_winpath_rfrom;
+reflectionFuncDecl Path_winloc_rfrom;
+reflectionFuncDecl Path_winfulloc_rfrom;
 
 reflectionFuncDecl Path_path_rto;
 reflectionFuncDecl Path_filename_rto;
 reflectionFuncDecl Path_unit_rto;
 reflectionFuncDecl Path_location_rto;
+reflectionFuncDecl Path_fullloc_rto;
 reflectionFuncDecl Path_file_rto;
 reflectionFuncDecl Path_extension_rto;
+
 CoreObject* PathObjectFactory( const CoreClass *cr, void *path, bool );
 
 
@@ -518,10 +560,10 @@ CoreObject* PathObjectFactory( const CoreClass *cr, void *path, bool );
 FALCON_FUNC  URI_init ( ::Falcon::VMachine *vm );
 FALCON_FUNC  URI_encode ( ::Falcon::VMachine *vm ); // static
 FALCON_FUNC  URI_decode ( ::Falcon::VMachine *vm ); // static
+FALCON_FUNC  URI_toString ( ::Falcon::VMachine *vm );
 FALCON_FUNC  URI_getFields ( ::Falcon::VMachine *vm );
 FALCON_FUNC  URI_setFields ( ::Falcon::VMachine *vm );
-reflectionFuncDecl URI_uri_rfrom;
-reflectionFuncDecl URI_uri_rto;
+
 
 /** Table class */
 FALCON_FUNC  Table_init ( ::Falcon::VMachine *vm );
@@ -535,6 +577,7 @@ FALCON_FUNC  Table_back ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_first ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_last ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_get ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Table_set ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_columnPos ( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  Table_columnData ( ::Falcon::VMachine *vm );
@@ -542,6 +585,7 @@ FALCON_FUNC  Table_columnData ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_find ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_insert ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_remove ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Table_append ( ::Falcon::VMachine *vm );
 
 FALCON_FUNC  Table_setColumn ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Table_insertColumn ( ::Falcon::VMachine *vm );
@@ -564,38 +608,42 @@ FALCON_FUNC  Tokenizer_rewind ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Tokenizer_nextToken ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Tokenizer_next ( ::Falcon::VMachine *vm );
 FALCON_FUNC  Tokenizer_token ( ::Falcon::VMachine *vm );
+FALCON_FUNC  Tokenizer_hasCurrent ( ::Falcon::VMachine *vm );
 
 #define TOKENIZER_OPT_GRROUPSEP 1
 #define TOKENIZER_OPT_BINDSEP 2
 #define TOKENIZER_OPT_TRIM 4
+#define TOKENIZER_OPT_RSEP 8
+#define TOKENIZER_OPT_WSISTOK 16
 
-class UriObject: public CRObject
+
+class UriObject: public CoreObject
 {
 public:
-   UriObject( const CoreClass *genr, URI* uri, bool bSerial ):
-      CRObject( genr, bSerial )
-   {
-      if ( uri == 0 )
-         uri = new URI();
-      setUserData( uri );
-      reflectFrom( uri );
-   }
+   UriObject( const CoreClass *gen ):
+      CoreObject( gen )
+   {}
+
+   UriObject( const CoreClass *gen, const URI& uri ):
+      CoreObject( gen ),
+      m_uri( uri )
+   {}
 
    UriObject( const UriObject &other );
    virtual ~UriObject();
-   virtual CoreObject *clone() const;
+   virtual UriObject *clone() const;
    virtual bool setProperty( const String &prop, const Item &value );
-   virtual void reflectFrom( void *user_data );
-   virtual void reflectTo( void *user_data ) const ;
+   virtual bool getProperty( const String &prop, Item &value ) const;
 
-   URI* getUri() const { return static_cast<URI*>( m_user_data ); }
+   const URI& uri() const { return m_uri; }
+   URI& uri() { return m_uri; }
+
+   static CoreObject* factory( const CoreClass *cr, void *uri, bool );
+private:
+   URI m_uri;
 };
 
 
-CoreObject* UriObjectFactory( const CoreClass *cr, void *uri, bool );
-
-
-
 /** Class used to manage the file stats.
    The FileStat reflection object provides three reflected timestamps
    in its object body.
@@ -641,7 +689,7 @@ public:
    virtual ~FileStatObject();
 
    virtual void gcMark( uint32 mark );
-   virtual CoreObject* clone() const;
+   virtual FileStatObject* clone() const;
 
    InnerData* getInnerData() const { return (InnerData*) m_user_data; }
 };
@@ -649,6 +697,56 @@ public:
 CoreObject* FileStatObjectFactory( const CoreClass *cls, void *user_data, bool bDeserializing );
 CoreObject* PathObjectFactory( const CoreClass *me, void *uri, bool dyn );
 
+/**********************************************
+ * Carrier of complex number objects.
+ */
+
+class CoreComplex : public CoreObject
+{
+   Complex m_complex;
+
+public:
+
+   CoreComplex ( const CoreClass *cls ):
+       CoreObject( cls )
+   { }
+
+   CoreComplex ( const Complex &origin, const CoreClass* cls ):
+      CoreObject( cls ),
+      m_complex( origin )
+   {}
+
+   CoreComplex ( const CoreComplex &other ):
+      CoreObject( other ),
+      m_complex( other.m_complex )
+      {}
+
+   virtual ~CoreComplex ();
+
+   virtual void gcMark( uint32 ) { }
+   virtual CoreObject *clone( void ) const { return new CoreComplex( *this ); }
+
+   virtual bool hasProperty( const String &key ) const;
+   virtual bool setProperty( const String &key, const Item &ret );
+   virtual bool getProperty( const String &key, Item &ret ) const;
+
+   const Complex &complex() const { return m_complex; }
+   Complex &complex() { return m_complex; }
+};
+
+CoreObject *Complex_Factory( const CoreClass *cls, void *, bool );
+
+FALCON_FUNC  Complex_init( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_toString( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_abs( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_conj( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_add__( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_sub__( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_mul__( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_div__( ::Falcon::VMachine *vm );
+FALCON_FUNC  Complex_compare( ::Falcon::VMachine *vm );
+
+
 }}
 
 #endif
