@@ -27,6 +27,7 @@
 #include <falcon/fassert.h>
 #include <falcon/eng_messages.h>
 #include <falcon/poopseq.h>
+#include <falcon/garbagepointer.h>
 
 namespace Falcon {
 namespace core {
@@ -772,9 +773,42 @@ FALCON_FUNC  mth_dictBest( ::Falcon::VMachine *vm )
    @brief Appends elements to this dictionary through a filter.
    @param source A sequence, a range or a callable generating items.
    @optparam filter A filtering function receiving one item at a time.
-   @return This array.
+   @return This dictionary.
 
    Please, see the description of @a Sequence.comp.
+
+   When the target sequence (this item) is a dictionary, each element that is to be
+   appended must be exactly an array with two items; the first will be used as a key,
+   and the second as the relative value.
+
+   For example:
+
+   @code
+   dict = [=>].comp(
+      // the source
+      .[ 'bananas' 'skip me' 'apples' 'oranges' '<end>' 'melons' ],
+      // the filter
+      { element, dict =>
+        if " " in element: return oob(1)
+        if "<end>" == element: return oob(0)
+        return [ "A" / len(dict), element ]   // (1)
+      }
+   )
+   @endcode
+
+   The element generated by the filter is a 2 element array, which is then
+   stored in the dictionary as a pair of key-value items.
+
+   @note In the example, the expression marked with (1) "A"/len(dict)
+   causes the current number of elements in the dictionary to be added to the UNICODE
+   value of the "A" letter; so the generated key will be "A" when the dictionary
+   has zero elements, "B" when it has one element and so on.
+
+   If the dictionary is blessed, then it is treated as an object, and instead of
+   adding directly the pair of key/value items, it's @b append method is repeatedly
+   called with the generated item as its parameter.
+   In this case, the type and length of each element is not relevant.
+
    @see Sequence.comp
 */
 
@@ -796,16 +830,102 @@ FALCON_FUNC  Dictionary_comp ( ::Falcon::VMachine *vm )
    if ( dict->isBlessed() )
    {
       // this will throw if dict has not "append"
-      PoopSeq seq( vm, dict );
-      seq.comprehension( vm, i_gen, i_check );
+      PoopSeq *seq = new PoopSeq( vm, dict );
+      vm->pushParam( new GarbagePointer( seq ) );
+      seq->comprehension_start( vm, dict, i_check );
    }
    else {
-      dict->items().comprehension( vm, i_gen, i_check );
+      dict->items().comprehension_start( vm, dict, i_check );
    }
 
-   vm->retval( vm->self() );
+   vm->pushParam( i_gen );
 }
 
+/*#
+   @method mcomp Dictionary
+   @brief Appends elements to this dictionary through a filter.
+   @param ... One or more sequences, ranges or callables generating items.
+   @optparam filter A filtering function receiving one item at a time.
+   @return This dictionary.
+
+   Please, see the description of @a Sequence.comp, and the general @a Dictionary.comp
+   for dictioanry-specific notes.
+
+   @see Sequence.mcomp
+*/
+
+FALCON_FUNC  Dictionary_mcomp ( ::Falcon::VMachine *vm )
+{
+   // Save the parameters as the stack may change greatly.
+   CoreDict* dict = vm->self().asDict();
+   StackFrame* current = vm->currentFrame();
+
+   // if this is a blessed dictionary, we must use the append method.
+   if ( dict->isBlessed() )
+   {
+      // this will throw if dict has not "append"
+      PoopSeq *seq = new PoopSeq( vm, dict );
+      vm->pushParam( new GarbagePointer( seq ) );
+      seq->comprehension_start( vm, dict, Item() );
+   }
+   else {
+      dict->items().comprehension_start( vm, dict, Item() );
+   }
+
+   for( uint32 i = 0; i < current->m_param_count; ++i )
+   {
+      vm->pushParam( current->m_params[i] );
+   }
+}
+
+/*#
+   @method mfcomp Dictionary
+   @brief Appends elements to this dictionary through a filter.
+   @param filter A filter function receiving each element before its insertion, or nil.
+   @param ... One or more sequences, ranges or callables generating items.
+   @optparam filter A filtering function receiving one item at a time.
+   @return This dictionary.
+
+   Please, see the description of @a Sequence.comp, and the general @a Dictionary.comp
+   for dictioanry-specific notes.
+
+   @see Sequence.mfcomp
+*/
+
+FALCON_FUNC  Dictionary_mfcomp ( ::Falcon::VMachine *vm )
+{
+   if ( vm->param(0) == 0 )
+   {
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+         .extra( "C, ..." ) );
+   }
+
+   // Save the parameters as the stack may change greatly.
+   CoreDict* dict = vm->self().asDict();
+   StackFrame* current = vm->currentFrame();
+
+   Item i_check = vm->param(0) == 0 ? Item(): *vm->param(10);
+
+   // if this is a blessed dictionary, we must use the append method.
+   if ( dict->isBlessed() )
+   {
+      // this will throw if dict has not "append"
+      PoopSeq *seq = new PoopSeq( vm, dict );
+      vm->pushParam( new GarbagePointer( seq ) );
+      seq->comprehension_start( vm, dict, i_check );
+   }
+   else {
+      dict->items().comprehension_start( vm, dict, i_check );
+   }
+
+   for( uint32 i = 1; i < current->m_param_count; ++i )
+   {
+      vm->pushParam( current->m_params[i] );
+   }
+}
+
+
+
 }
 }
 
