@@ -22,18 +22,14 @@
 #include <windows.h>
 #endif
 
-/*#
-   
-*/
-
 namespace Falcon {
 namespace core {
 
 
 /*#
    @function seconds
-   @inset core_general_purpose
-   @brief Returns the number of seconds since the “epoch” as reported by the system.
+   @ingroup general_purpose
+   @brief Returns the number of seconds since the "epoch" as reported by the system.
    @return The number of seconds and fractions of seconds in a floating point value.
 
    Actually, this function returns a floating point number which represents seconds
