@@ -210,7 +210,7 @@ FALCON_FUNC mth_strFront( VMachine *vm )
 
 /*#
    @function strBack
-   @brief Returns the first character in a string.
+   @brief Returns the last character(s) in a string.
    @param str The string on which to operate.
    @optparam count Number of characters to be taken (defaults to 1).
    @optparam remove If true, remove also the character.
@@ -220,7 +220,7 @@ FALCON_FUNC mth_strFront( VMachine *vm )
    This function returns a string containing one character from the end of @b str,
    or eventually more characters in case a number > 1 is specified in @b count.
 
-   If @b remove is true, then the character is removed and the string is shrinked.
+   If @b remove is true, then the characters are removed and the string is shrinked.
    @see String.front
 
    If @b numeric is true,
@@ -325,12 +325,12 @@ FALCON_FUNC mth_strBack( VMachine *vm )
    at worst a single element containing a copy of the whole string passed as a
    parameter.
 
-   Contrarily to @a strSplit, this function will "eat up" adjiacent token. While
+   Contrarily to @a strSplit, this function will "eat up" adjacent tokens. While
    @a strSplit is more adequate to parse field-oriented strings (as i.e.
    colon separated fields in configuration files) this function is best employed
    in word extraction.
 
-   @note this function is equivalent to the fbom method String.splittr
+   @note this function is equivalent to the FBOM method String.splittr
 
    @note See @a Tokenizer for a more adequate function to scan extensively
    wide strings.
@@ -379,7 +379,7 @@ FALCON_FUNC  mth_strSplitTrimmed ( ::Falcon::VMachine *vm )
 
    limit = count == 0 ? 0xffffffff: (int32) count->forceInteger();
 
-   // Parameter estraction.
+   // Parameter extraction.
    String *tg_str = target->asString();
    uint32 tg_len = target->asString()->length();
 
@@ -405,6 +405,7 @@ FALCON_FUNC  mth_strSplitTrimmed ( ::Falcon::VMachine *vm )
 
    uint32 pos = 0;
    uint32 last_pos = 0;
+   bool lastIsEmpty = false;
    // scan the string
    while( limit > 1 && pos <= tg_len - sp_len  )
    {
@@ -421,6 +422,9 @@ FALCON_FUNC  mth_strSplitTrimmed ( ::Falcon::VMachine *vm )
          // put the item in the array.
          uint32 splitend = pos - sp_len;
          retarr->append( new CoreString( String( *tg_str, last_pos, splitend ) ) );
+
+         lastIsEmpty = (last_pos + 1 >= splitend);
+
          last_pos = pos;
          limit--;
          // skip matching pattern
@@ -441,9 +445,9 @@ FALCON_FUNC  mth_strSplitTrimmed ( ::Falcon::VMachine *vm )
    }
 
    // Residual element?
-   if ( limit >= 1 || last_pos < tg_len ) {
-      uint32 splitend = tg_len;
-      retarr->append( new CoreString( String( *tg_str, last_pos, splitend ) ) );
+   // -- but only if we didn't already put a "" in the array
+   if ( limit >= 1 && ! lastIsEmpty ) {
+      retarr->append( new CoreString( String( *tg_str, last_pos, (uint32) tg_len ) ) );
    }
 
    vm->retval( retarr );
@@ -1748,6 +1752,191 @@ FALCON_FUNC  mth_strCmpIgnoreCase ( ::Falcon::VMachine *vm )
    vm->retval( 0 );
 }
 
+
+inline void internal_escape( int mode, VMachine* vm )
+{
+   Item *i_string;
+   Item *i_onplace;
+
+   // Parameter checking;
+   if( vm->self().isMethodic() )
+   {
+      i_string = &vm->self();
+      i_onplace = vm->param(0);
+   }
+   else
+   {
+      i_string = vm->param(0);
+      i_onplace = vm->param(1);
+      if ( i_string == 0 || ! i_string->isString() )
+         {
+            throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+                  .origin( e_orig_runtime )
+                  .extra( "S,B" ) );
+         }
+   }
+
+   String* str;
+
+   if ( mode != 2 && i_onplace != 0 && i_onplace->isTrue() )
+   {
+      str = i_string->asString();
+
+   }
+   else
+   {
+      str = new CoreString( *i_string->asString() );
+   }
+
+   switch( mode )
+   {
+   case 0: //esq
+      str->escapeQuotes();
+      break;
+
+   case 1: //unesq
+      str->unescapeQuotes();
+      break;
+
+   case 2: //escape
+   {
+      CoreString* other = new CoreString( str->size() );
+      if ( i_onplace != 0 && i_onplace->isTrue() ) // actually it means "complete"
+         str->escapeFull( *other );
+      else
+         str->escape( *other );
+
+      vm->retval( other );
+   }
+   return;
+
+
+   case 3: //unescape
+      str->unescape();
+      break;
+   }
+
+   vm->retval( str );
+}
+
+/*#
+   @function strEsq
+   @brief Escape quotes in the given string.
+   @param string the String to be escaped.
+   @optparam inplace if true, the source string is modified, saving memory.
+   @return A new escaped string, if @b inplace is not given, or the @b string parameter
+           if @b inplace is true.
+
+
+   @see String.esq
+   @see strUnesq
+*/
+/*#
+   @method esq String
+   @brief Escapes the quotes in this string.
+   @optparam inplace if true, the source string is modified, saving memory.
+   @return A new escaped string, if @b inplace is not given, or this same string
+           if @b inplace is true.
+
+   @see String.unesq
+   @see strEsq
+*/
+
+FALCON_FUNC  mth_strEsq ( ::Falcon::VMachine *vm )
+{
+   internal_escape( 0, vm );
+}
+
+/*#
+   @function strUnesq
+   @brief Unescape the quotes in given string.
+   @param string the String to be unescaped.
+   @optparam inplace if true, the source string is modified, saving memory.
+   @return A new unescaped string, if @b inplace is not given, or the @b string parameter
+           if @b inplace is true.
+
+   This function transforms all the occourences of '\\"' and '\\'' into a double or
+   single quote, leaving all the other special escape sequences untouched.
+
+   @see String.unesq
+   @see strEsq
+*/
+/*#
+   @method unesq String
+   @brief Escapes the quotes in this string.
+   @optparam inplace if true, the source string is modified, saving memory.
+   @return A new escaped string, if @b inplace is not given, or this same string
+           if @b inplace is true.
+
+   @see String.esq
+   @see strUnesq
+*/
+
+FALCON_FUNC  mth_strUnesq ( ::Falcon::VMachine *vm )
+{
+   internal_escape( 1, vm );
+}
+
+
+/*#
+   @function strEscape
+   @brief Escape quotes and special characters in the string
+   @param string the String to be escaped.
+   @optparam full If true, characters above UNICODE 127 are escaped as well.
+   @return A new escaped string.
+
+
+   @see String.esq
+   @see strUnesq
+*/
+/*#
+   @method escape String
+   @brief Escapes all the special characters in the string.
+   @optparam full If true, characters above UNICODE 127 are escaped as well.
+   @return A new escaped string.
+
+   @see String.esq
+   @see strEsq
+   @see strUnescape
+*/
+
+FALCON_FUNC  mth_strEscape ( ::Falcon::VMachine *vm )
+{
+   internal_escape( 2, vm );
+}
+
+/*#
+   @function strUnescape
+   @brief Unescape quotes and special characters in the string
+   @param string the String to be escaped.
+   @optparam inplace if true, the source string is modified, saving memory.
+   @return A new unescaped string, if @b inplace is not given, or the @b string parameter
+           if @b inplace is true.
+
+
+
+   @see String.esq
+   @see strUnesq
+   @see String.unescape
+*/
+/*#
+   @method unescape String
+   @brief Unescapes all the special characters in the string.
+   @optparam inplace if true, the source string is modified, saving memory.
+   @return A new unescaped string, if @b inplace is not given, or the @b string parameter
+           if @b inplace is true.
+
+
+   @see String.esq
+   @see strEsq
+   @see strEscape
+*/
+
+FALCON_FUNC  mth_strUnescape ( ::Falcon::VMachine *vm )
+{
+   internal_escape( 3, vm );
+}
+
 /*#
    @function strWildcardMatch
    @brief Perform an old-style file-like jolly-based wildcard match.
@@ -1824,106 +2013,8 @@ FALCON_FUNC  mth_strWildcardMatch ( ::Falcon::VMachine *vm )
    // The first is the wildcard, the second is the matched thing.
    String *cfr = s1_itm->asString();
    String *wcard = s2_itm->asString();
-   uint32 wpos = 0, wlen = wcard->length();
-   uint32 cpos = 0, clen = cfr->length();
 
-   uint32 wstarpos = 0xFFFFFFFF;
-
-
-   while ( cpos < clen )
-   {
-      if( wpos == wlen )
-      {
-         // we have failed the match; but if we had a star, we
-         // may roll back to the starpos and try to match the
-         // rest of the string
-         if ( wstarpos != 0xFFFFFFFF )
-         {
-            wpos = wstarpos;
-         }
-         else {
-            // no way, we're doomed.
-            break;
-         }
-      }
-
-      uint32 wchr = wcard->getCharAt( wpos );
-      uint32 cchr = cfr->getCharAt( cpos );
-
-      switch( wchr )
-      {
-         case '?': // match any character
-            wpos++;
-            cpos++;
-         break;
-
-         case '*':
-         {
-            // mark for restart in case of bad match.
-            wstarpos = wpos;
-
-            // match till the next character
-            wpos++;
-            // eat all * in a row
-            while( wpos < wlen )
-            {
-               wchr = wcard->getCharAt( wpos );
-               if ( wchr != '*' )
-                  break;
-               wpos++;
-            }
-
-            if ( wpos == wlen )
-            {
-               // we have consumed all the chars
-               cpos = clen;
-               break;
-            }
-
-
-            //eat up to next character
-            while( cpos < clen )
-            {
-               cchr = cfr->getCharAt( cpos );
-               if ( cchr == wchr )
-                  break;
-               cpos ++;
-            }
-
-            // we have eaten up the same char? --  then advance also wpos to prepare next loop
-            if ( cchr == wchr )
-            {
-               wpos++;
-               cpos++;
-            }
-            // else, everything must stay as it is, so cpos == clen but wpos != wlen causing fail.
-         }
-         break;
-
-         default:
-            if ( cchr == wchr ||
-                  ( bIcase && cchr < 128 && wchr < 128 && (cchr | 32) == (wchr | 32) )
-               )
-            {
-               cpos++;
-               wpos++;
-            }
-            else
-            {
-               // can we retry?
-               if ( wstarpos != 0xFFFFFFFF )
-                  wpos = wstarpos;
-               else {
-                  // check failed -- we're doomed
-                  vm->retval( false );
-                  return;
-               }
-            }
-      }
-   }
-
-   // at the end of the loop, the match is ok only if both the cpos and wpos are at the end
-   vm->regA().setBoolean( wpos == wlen && cpos == clen );
+   vm->regA().setBoolean( cfr->wildcardMatch( *wcard, bIcase ) );
 }
 
 /*#
