@@ -101,16 +101,19 @@ bool fal_stats( const String &f, FileStat &sts )
 
    UnixSystemTime mtime( fs.st_mtime );
 
-   sts.m_atime = new TimeStamp();
+   if (sts.m_atime == 0 )
+      sts.m_atime = new TimeStamp();
    mtime.m_time_t = fs.st_atime;
    sts.m_atime->fromSystemTime( mtime );    /* time of last access */
 
-   sts.m_ctime = new TimeStamp();
+   if (sts.m_ctime == 0 )
+      sts.m_ctime = new TimeStamp();
    mtime.m_time_t = fs.st_ctime;
    sts.m_ctime->fromSystemTime( mtime );     /* time of last change */
 
    // copy last change time to last modify time
-   sts.m_mtime = new TimeStamp();
+   if (sts.m_mtime == 0 )
+      sts.m_mtime = new TimeStamp();
    sts.m_mtime->fromSystemTime( mtime );
 
    return true;
@@ -128,6 +131,44 @@ bool fal_mkdir( const String &f, int32 &fsStatus )
    return false;
 }
 
+bool fal_mkdir( const String &strName, int32 &fsError, bool descend )
+{
+   if ( descend )
+   {
+      // find /.. sequences
+      uint32 pos = strName.find( "/" );
+      while( true )
+      {
+         String strPath( strName, 0, pos );
+
+         // stat the file
+         FileStat fstats;
+         // if the file exists...
+         if ( (! Sys::fal_stats( strPath, fstats )) ||
+              fstats.m_type != FileStat::t_dir )
+         {
+            // if it's not a directory, try to create the directory.
+            if ( ! Sys::fal_mkdir( strPath, fsError ) )
+               return false;
+         }
+
+         // last loop?
+         if ( pos == String::npos )
+            break;
+
+         pos = strName.find( "/", pos + 1 );
+       }
+
+   }
+   else
+   {
+      // Just one try; succeed or fail
+      return Sys::fal_mkdir( strName, fsError );
+   }
+   
+   return true;
+}
+
 bool fal_rmdir( const String &f, int32 &fsStatus )
 {
    AutoCString filename( f );
@@ -181,38 +222,47 @@ bool fal_chdir( const String &f, int32 &fsStatus )
 
 bool fal_getcwd( String &fname, int32 &fsError )
 {
-   uint32 pwdSize = 64;
 
-   char *buffer = (char *) memAlloc( 64 );
+   char buf[256];
+   uint32 pwdSize = 256;
+   char *buffer = buf;
    char *bufret;
 
    while ( (bufret = ::getcwd( buffer, pwdSize )) == 0 && errno == ERANGE ) {
-      pwdSize += 64;
+      pwdSize += 256;
+      if ( buffer != buf )
+         memFree( buffer );
       buffer = ( char * ) memAlloc( pwdSize );
    }
 
-   if ( bufret == 0 )
+   fsError = errno;
+   bool val;
+   if ( bufret != 0 )
    {
-      memFree( buffer );
-      fsError = errno;
-      return 0;
+      val = true;
+      fname.fromUTF8( bufret );
    }
+   else
+      val = false;
 
-   fname.bufferize( buffer );
-   memFree( buffer );
-   return true;
+   if ( buffer != buf )
+      memFree( buffer );
+
+   return val;
 }
 
 bool fal_chmod( const String &fname, uint32 mode )
 {
    AutoCString filename( fname );
    bool ret = ::chmod( filename.c_str(), mode ) == 0;
+   return ret;
 }
 
 bool fal_chown( const String &fname, int32 owner )
 {
    AutoCString filename( fname );
-   bool ret = ::chown( filename.c_str(), owner , -1 ) == 0;
+   bool ret = ::chown( filename.c_str(), owner , (gid_t) -1 ) == 0;
+   return ret;
 }
 
 bool fal_readlink( const String &fname, String &link )
@@ -223,7 +273,7 @@ bool fal_readlink( const String &fname, String &link )
 
    if ( ( len = readlink( filename.c_str(), buf, sizeof(buf) - 1 ) ) != -1) {
       buf[len] = '\0';
-      link.bufferize( buf );
+      link.fromUTF8( buf );
       return true;
    }
    return false;
@@ -245,7 +295,7 @@ bool fal_writelink( const String &fname, const String &link )
 bool fal_chgrp( const String &fname, int32 owner )
 {
    AutoCString filename( fname );
-   bool ret = ::chown( filename.c_str(), -1, owner ) == 0;
+   bool ret = ::chown( filename.c_str(), (uid_t) -1, owner ) == 0;
    return ret;
 }
 
@@ -259,7 +309,7 @@ bool fal_chgrp( const String &fname, int32 owner )
       return 0;
    }
 
-   return new DirEntry_unix( dir );
+   return new DirEntry_unix( p, dir );
 }
 
 void fal_closeDir( ::Falcon::DirEntry *entry )
@@ -283,7 +333,7 @@ bool DirEntry_unix::read( String &res )
       return false;
    }
 
-   res.bufferize( d->d_name );
+   res.fromUTF8( d->d_name );
    return true;
 }
 
