@@ -41,6 +41,11 @@ const String &errorDesc( int code )
 
 String &TraceStep::toString( String &target ) const
 {
+   if ( m_modpath.size() )
+   {
+      target += "\"" + m_modpath + "\" "; 
+   }
+
    target += m_module + "." + m_symbol + ":";
    target.writeNumber( (int64) m_line );
    target += "(PC:";
@@ -173,6 +178,7 @@ String &Error::heading( String &target ) const
    case e_orig_runtime: target += "RT"; break;
    case e_orig_mod: target += "MD"; break;
    case e_orig_script: target += "SS"; break;
+   default: target += "??";
    }
 
    uint32 ecode = (uint32) m_errorCode;
@@ -252,13 +258,18 @@ String &Error::heading( String &target ) const
    return target;
 }
 
-
 void Error::addTrace( const String &module, const String &symbol, uint32 line, uint32 pc )
 {
    m_steps.pushBack( new TraceStep( module, symbol, line, pc ) );
    m_stepIter = m_steps.begin();
 }
 
+void Error::addTrace( const String &module, const String &modpath, const String &symbol, uint32 line, uint32 pc )
+{
+   m_steps.pushBack( new TraceStep( module, modpath, symbol, line, pc ) );
+   m_stepIter = m_steps.begin();
+}
+
 void Error::appendSubError( Error *error )
 {
    if ( m_LastNextError == 0 )
@@ -366,12 +377,10 @@ void Error_origin_rfrom(CoreObject *instance, void *userData, Item &property, co
       case e_orig_script: origin = "script"; break;
       case e_orig_runtime: origin = "runtime"; break;
       case e_orig_mod: origin = "module"; break;
+      default: origin = "unknown";
    }
 
-   if ( !property.isString() || origin != *property.asString() )
-   {
-      property = new CoreString( origin );
-   }
+   property = new CoreString( origin );
 }
 
 void Error_module_rfrom(CoreObject *instance, void *userData, Item &property, const PropEntry& )
@@ -548,7 +557,7 @@ void ErrorObject::gcMark( uint32 mark )
 }
 
 
-CoreObject *ErrorObject::clone() const
+ErrorObject *ErrorObject::clone() const
 {
    return new ErrorObject( m_generatedBy, getError() );
 }
@@ -557,12 +566,11 @@ CoreObject *ErrorObject::clone() const
 // Factory function
 //
 
-CoreObject* ErrorObjectFactory( const CoreClass *cls, void *user_data, bool bDeserial )
+CoreObject* ErrorObjectFactory( const CoreClass *cls, void *user_data, bool )
 {
       return new ErrorObject( cls, (Error *) user_data );
 }
 
-}
-}
+}} // namespace Falcon::core 
 
 /* end of error.cpp */
