@@ -33,14 +33,21 @@
 #include <math.h>
 
 #include <stdio.h>
+
 namespace Falcon {
 
 // Generic fail
 void co_fail()
 {
-   throw new TypeError( ErrorParam( e_invop ).extra( "op" ) );
+   throw new TypeError( ErrorParam( e_invop ) );
 }
 
+void co_fail_unb()
+{
+   throw new TypeError( ErrorParam( e_invop_unb ) );
+}
+
+
 //=============================================================
 // Add
 //
@@ -121,12 +128,12 @@ void co_dict_add( const Item& first, const Item& second, Item& third )
    const Item *op2 = second.dereference();
    Item mth;
 
-   if ( source->getMethod( "add__", mth ) )
+   if ( source->getMethod( OVERRIDE_OP_ADD, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm != 0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
@@ -181,19 +188,19 @@ void co_object_add( const Item& first, const Item& second, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "add__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_ADD, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm != 0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "add__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_ADD ) );
 }
 
 void co_ref_add( const Item& first, const Item& second, Item& third )
@@ -303,12 +310,12 @@ void co_dict_sub( const Item& first, const Item& second, Item& third )
    const Item *op2 = second.dereference();
    Item mth;
 
-   if ( source->getMethod( "sub__", mth ) )
+   if ( source->getMethod( OVERRIDE_OP_SUB, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm != 0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
@@ -338,19 +345,19 @@ void co_object_sub( const Item& first, const Item& second, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "sub__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_SUB, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm != 0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "sub__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_SUB ) );
 }
 
 void co_ref_sub( const Item& first, const Item& second, Item& third )
@@ -449,19 +456,19 @@ void co_dict_mul( const Item& first, const Item& second, Item& third )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "mul__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_MUL, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "mul__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_MUL ) );
 }
 
 
@@ -470,19 +477,19 @@ void co_object_mul( const Item& first, const Item& second, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "mul__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_MUL, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "mul__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_MUL ) );
 }
 
 
@@ -600,19 +607,19 @@ void co_dict_div( const Item& first, const Item& second, Item& third )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "div__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_DIV, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "div__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_DIV ) );
 }
 
 
@@ -621,19 +628,19 @@ void co_object_div( const Item& first, const Item& second, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "div__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_DIV, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "div__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_DIV ) );
 }
 
 void co_ref_div( const Item& first, const Item& second, Item& third )
@@ -744,19 +751,19 @@ void co_dict_mod( const Item& first, const Item& second, Item& third )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "mod__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_MOD, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "mod__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_MOD ) );
 }
 
 
@@ -765,19 +772,19 @@ void co_object_mod( const Item& first, const Item& second, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "mod__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_MOD, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "mod__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_MOD ) );
 }
 
 void co_ref_mod( const Item& first, const Item& second, Item& third )
@@ -875,19 +882,19 @@ void co_dict_pow( const Item& first, const Item& second, Item& third )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "pow__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_POW, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "pow__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_POW ) );
 }
 
 
@@ -896,19 +903,19 @@ void co_object_pow( const Item& first, const Item& second, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "pow__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_POW, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *second.dereference() );
+         vm->pushParam( *second.dereference() );
          vm->callItemAtomic( mth, 1 );
          third = vm->regA();
          return;
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "pow__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_POW ) );
 }
 
 void co_ref_pow( const Item& first, const Item& second, Item& third )
@@ -938,7 +945,7 @@ void co_dict_neg( const Item& first, Item& tgt )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "neg__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_NEG, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -949,7 +956,7 @@ void co_dict_neg( const Item& first, Item& tgt )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "neg__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_NEG ) );
 }
 
 
@@ -958,7 +965,7 @@ void co_object_neg( const Item& first, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "neg__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_NEG, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -969,7 +976,7 @@ void co_object_neg( const Item& first, Item& third )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "neg__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_NEG ) );
 }
 
 void co_ref_neg( const Item& first, Item &tgt )
@@ -1001,7 +1008,7 @@ void co_dict_inc( Item& first, Item &target )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "inc__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_INC, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1012,7 +1019,7 @@ void co_dict_inc( Item& first, Item &target )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "inc__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_INC ) );
 }
 
 
@@ -1021,7 +1028,7 @@ void co_object_inc( Item& first, Item &target )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "inc__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_INC, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1032,7 +1039,7 @@ void co_object_inc( Item& first, Item &target )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "inc__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_INC ) );
 }
 
 void co_ref_inc( Item& first, Item &target )
@@ -1064,7 +1071,7 @@ void co_dict_dec( Item& first, Item &target )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "dec__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_DEC, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm != 0 )
@@ -1075,7 +1082,7 @@ void co_dict_dec( Item& first, Item &target )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "dec__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_DEC ) );
 }
 
 
@@ -1084,7 +1091,7 @@ void co_object_dec( Item& first, Item &target )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "dec__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_DEC, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1095,7 +1102,7 @@ void co_object_dec( Item& first, Item &target )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "dec__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_DEC ) );
 }
 
 void co_ref_dec( Item& first, Item &target )
@@ -1127,7 +1134,7 @@ void co_dict_incpost( Item& first, Item& third )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "incpost__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_INCPOST, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1138,7 +1145,7 @@ void co_dict_incpost( Item& first, Item& third )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "incpost__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_INCPOST ) );
 }
 
 
@@ -1148,7 +1155,7 @@ void co_object_incpost( Item& first, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "incpost__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_INCPOST, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1159,7 +1166,7 @@ void co_object_incpost( Item& first, Item& third )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "incpost__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_INCPOST ) );
 }
 
 void co_ref_incpost( Item& first, Item &tgt )
@@ -1191,7 +1198,7 @@ void co_dict_decpost( Item& first, Item& third )
    CoreDict *self = first.asDict();
 
    Item mth;
-   if ( self->getMethod( "decpost__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_DECPOST, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1202,7 +1209,7 @@ void co_dict_decpost( Item& first, Item& third )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "decpost__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_DECPOST ) );
 }
 
 
@@ -1211,7 +1218,7 @@ void co_object_decpost( Item& first, Item& third )
    CoreObject *self = first.asObjectSafe();
 
    Item mth;
-   if ( self->getMethod( "decpost__", mth ) )
+   if ( self->getMethod( OVERRIDE_OP_DECPOST, mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
@@ -1222,7 +1229,7 @@ void co_object_decpost( Item& first, Item& third )
       }
    }
 
-   throw new TypeError( ErrorParam( e_invop ).extra( "decpost__" ) );
+   throw new TypeError( ErrorParam( e_invop ).extra( OVERRIDE_OP_DECPOST ) );
 }
 
 void co_ref_decpost( Item& first, Item &tgt )
@@ -1238,9 +1245,16 @@ void co_ref_decpost( Item& first, Item &tgt )
 
 int co_nil_compare( const Item& first, const Item& second )
 {
-   if ( second.isReference() )
+   switch( second.type() )
+   {
+   case FLC_ITEM_REFERENCE:
       return co_nil_compare( first, second.asReference()->origin() );
 
+   case FLC_ITEM_UNB:
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
+
    return first.type() - second.type();
 }
 
@@ -1259,6 +1273,10 @@ int co_bool_compare( const Item& first, const Item& second )
 
       case FLC_ITEM_REFERENCE:
          return co_bool_compare( first, second.asReference()->origin() );
+
+      case FLC_ITEM_UNB:
+         const_cast<Item*>(&second)->copy( first );
+         return 0;
    }
 
    // bool is always lower
@@ -1274,13 +1292,23 @@ int co_int_compare( const Item& first, const Item& second )
          return 1;
 
       case FLC_ITEM_INT:
-         return (int)(first.asInteger() - second.asInteger()) ;
+      {
+         // converting the int to a 32 bit integer is not a simple matter
+         int64 v = (first.asInteger() - second.asInteger()) ;
+         if (v==0) return 0;
+         if ( v > 0 ) return 1;
+         return -1;
+      }
 
       case FLC_ITEM_NUM:
          return (int)(((numeric)first.asInteger()) - second.asNumeric());
 
       case FLC_ITEM_REFERENCE:
          return co_int_compare( first, second.asReference()->origin() );
+
+      case FLC_ITEM_UNB:
+         const_cast<Item*>(&second)->copy( first );
+         return 0;
       }
 
    // lower than the others
@@ -1312,6 +1340,10 @@ int co_num_compare( const Item& first, const Item& second )
 
       case FLC_ITEM_REFERENCE:
          return co_num_compare( first, second.asReference()->origin() );
+
+      case FLC_ITEM_UNB:
+         const_cast<Item*>(&second)->copy( first );
+         return 0;
     }
 
    // lower than the others
@@ -1331,29 +1363,33 @@ int co_range_compare( const Item& first, const Item& second )
 
       case FLC_ITEM_RANGE:
       {
-         int diff = (int)(first.asRangeStart() - second.asRangeStart());
+         int64 diff = first.asRangeStart() - second.asRangeStart();
          if ( diff != 0 )
-            return diff;
+            return (int)(diff>>32);
 
          // always greater
          diff =  (first.asRangeIsOpen() ? 1: 0) - (second.asRangeIsOpen() ? 1: 0);
 
          if ( diff != 0 )
-            return diff;
+            return (int)(diff);
 
          if ( first.asRangeIsOpen() )
             return 0;
 
-         diff = (int)(first.asRangeEnd() - second.asRangeEnd());
+         diff = first.asRangeEnd() - second.asRangeEnd();
          if ( diff != 0 )
-            return diff;
+            return (int)(diff>>32);
 
-         return (int)(first.asRangeStep() - second.asRangeStep());
+         return (int)((first.asRangeStep() - second.asRangeStep())>>32);
       }
       break;
 
       case FLC_ITEM_REFERENCE:
          return co_range_compare( first, second.asReference()->origin() );
+
+      case FLC_ITEM_UNB:
+         const_cast<Item*>(&second)->copy( first );
+         return 0;
    }
 
    return -1;
@@ -1361,13 +1397,18 @@ int co_range_compare( const Item& first, const Item& second )
 
 int co_lbind_compare( const Item& first, const Item& second )
 {
-   if ( second.isReference() )
+   switch( second.type() )
    {
+
+   case FLC_ITEM_REFERENCE:
       return co_lbind_compare( first, second.asReference()->origin() );
-   }
-   else if ( second.isLBind() )
-   {
+
+   case FLC_ITEM_LBIND:
       return (int)(first.asLBind() - second.asLBind());
+
+   case FLC_ITEM_UNB:
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
    }
 
    return first.type() - second.type();
@@ -1375,13 +1416,18 @@ int co_lbind_compare( const Item& first, const Item& second )
 
 int co_func_compare( const Item& first, const Item& second )
 {
-   if ( second.isReference() )
+   switch( second.type() )
    {
+
+   case FLC_ITEM_REFERENCE:
       return co_func_compare( first, second.asReference()->origin() );
-   }
-   else if ( second.isFunction() )
-   {
+
+   case FLC_ITEM_FUNC:
       return first.asFunction()->name().compare(second.asFunction()->name());
+
+   case FLC_ITEM_UNB:
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
    }
 
    return first.type() - second.type();
@@ -1397,6 +1443,11 @@ int co_gcptr_compare( const Item& first, const Item& second )
    {
       return (int)(first.asGCPointer() - second.asGCPointer());
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
 
    return first.type() - second.type();
 }
@@ -1412,6 +1463,11 @@ int co_string_compare( const Item& first, const Item& second )
    {
       return first.asString()->compare( *second.asString() );
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
 
    return first.type() - second.type();
 }
@@ -1425,7 +1481,12 @@ int co_array_compare( const Item& first, const Item& second )
    }
    else if ( second.isArray() )
    {
-      return (int)(first.asArray() - second.asArray());
+      return first.asArray()->compare( *second.asArray() );
+   }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
    }
 
    return (first.type() - second.type());
@@ -1437,13 +1498,19 @@ int co_dict_compare( const Item& first, const Item& second )
    CoreDict *self = first.asDict();
    const Item* psecond = second.dereference();
 
+   if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
+
    Item mth;
    if ( self->getMethod( "compare", mth ) )
    {
       VMachine *vm = VMachine::getCurrent();
       if ( vm !=  0 )
       {
-         vm->pushParameter( *psecond );
+         vm->pushParam( *psecond );
          vm->callItemAtomic( mth, 1 );
          if ( ! vm->regA().isNil() )
             return (int) vm->regA().forceInteger();
@@ -1453,7 +1520,7 @@ int co_dict_compare( const Item& first, const Item& second )
 
    if ( psecond->isDict() )
    {
-      return (int)(first.asDict() - psecond->asDict());
+      return first.asDict()->compare(psecond->asDict());
    }
 
    return first.type() - psecond->type();
@@ -1466,6 +1533,11 @@ int co_object_compare( const Item& first, const Item& second )
    {
       return co_object_compare( first, second.asReference()->origin() );
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
    else
    {
       CoreObject *self = first.asObjectSafe();
@@ -1478,7 +1550,7 @@ int co_object_compare( const Item& first, const Item& second )
          Item mth;
          if ( self->getMethod( "compare", mth ) )
          {
-            vm->pushParameter( second );
+            vm->pushParam( second );
             vm->callItemAtomic( mth, 1 );
             if ( vm->regA().isInteger() )
             {
@@ -1506,6 +1578,12 @@ int co_membuf_compare( const Item& first, const Item& second )
    {
       return (int)(first.asMemBuf() - second.asMemBuf());
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
+
 
    return (int)(first.type() - second.type());
 }
@@ -1526,6 +1604,12 @@ int co_clsmethod_compare( const Item& first, const Item& second )
    {
       return (int)(first.asMethodFunc() - second.asMethodFunc());
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
+
 
    return first.type() - second.type();
 }
@@ -1543,6 +1627,12 @@ int co_method_compare( const Item& first, const Item& second )
          return (int)(first.asMethodFunc() - second.asMethodFunc());
       return cp;
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
+
 
    return first.type() - second.type();
 }
@@ -1557,11 +1647,26 @@ int co_class_compare( const Item& first, const Item& second )
    {
       return (int)(first.asClass()->symbol()->name().compare(first.asClass()->symbol()->name()));
    }
+   else if ( second.isUnbound() )
+   {
+      const_cast<Item*>(&second)->copy( first );
+      return 0;
+   }
+
 
    return (int)(first.type() - second.type());
 }
 
 
+int co_unb_compare( const Item& first, const Item& second )
+{
+   if ( second.isReference() )
+      return co_unb_compare( first, second.asReference()->origin() );
+
+   const_cast<Item*>(&first)->copy( second );
+   return 0;
+}
+
 //=============================================================
 // Get Index
 //
@@ -1635,6 +1740,7 @@ void co_ref_getindex( const Item &item, const Item &idx, Item &result )
    item.asReference()->origin().getIndex( idx, result );
 }
 
+
 //=============================================================
 // Set Index
 //
@@ -2010,7 +2116,7 @@ void co_call_dict( const Item &itm, VMachine *vm, uint32 paramCount )
    CoreDict *self = itm.asDict();
 
    Item *mth;
-   if ( (mth = self->find( "call__" ) ) != 0 && mth->isCallable() )
+   if ( (mth = self->find( OVERRIDE_OP_CALL ) ) != 0 && mth->isCallable() )
    {
       mth->readyFrame( vm, paramCount );
       vm->self() = self;
@@ -2028,7 +2134,7 @@ void co_call_object( const Item &itm, VMachine *vm, uint32 paramCount )
    CoreObject *self = itm.asObjectSafe();
 
    Item mth;
-   if ( self->getProperty( "call__", mth ) && mth.isCallable() )
+   if ( self->getProperty( OVERRIDE_OP_CALL, mth ) && mth.isCallable() )
    {
       mth.readyFrame( vm, paramCount );
       vm->self() = self;
@@ -2046,6 +2152,26 @@ void co_call_method( const Item &itm, VMachine *vm, uint32 paramCount )
    itm.getMethodItem( vm->self() );
 }
 
+static bool __call_init_enter_last( VMachine *vm )
+{
+   vm->regA() = vm->self();
+   return false;
+}
+
+static bool __call_init_enter( VMachine *vm )
+{
+   Item initEnter;
+   if( vm->self().asObject()->getMethod( "__enter", initEnter ) )
+   {
+      vm->returnHandler(0);
+      vm->callFrame( initEnter, 0 );
+      return true;
+   }
+
+   vm->regA() =  vm->self();
+   return false;
+}
+
 void co_call_class( const Item &itm, VMachine *vm, uint32 paramCount )
 {
    CoreClass *cls = itm.asClass();
@@ -2061,6 +2187,18 @@ void co_call_class( const Item &itm, VMachine *vm, uint32 paramCount )
 
       // pop the stack
       vm->stack().resize( vm->stack().length() - paramCount );
+      if( cls->hasInitEnter() )
+      {
+         Item initEnter;
+         if( inst->getProperty( "__enter", initEnter ) && initEnter.isFunction() )
+         {
+            vm->prepareFrame( initEnter.asFunction(), 0 );
+            vm->self() = inst;
+            inst->gcMark( memPool->generation() );
+            vm->returnHandler( __call_init_enter_last );
+         }
+      }
+
       return;
    }
 
@@ -2070,8 +2208,12 @@ void co_call_class( const Item &itm, VMachine *vm, uint32 paramCount )
 
    // also return self; we need to tell it to the VM
    vm->requestConstruct();
-}
 
+   if ( cls->hasInitEnter() )
+   {
+      vm->returnHandler( __call_init_enter );
+   }
+}
 
 //=============================================================
 // Tables declaration
@@ -2472,7 +2614,7 @@ void* ReferenceCommOpsTable[] = {
    (void*) co_ref_setproperty,
 
    //call
-   (void*) co_call_uncallable
+   (void*) co_call_reference
 };
 
 void* ClsMethodCommOpsTable[] = {
@@ -2564,6 +2706,35 @@ void* ClassCommOpsTable[] = {
 };
 
 
+void* UnbCommOpsTable[] = {
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+
+   // Inc/dec
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+
+   // cfr
+   (void*) co_unb_compare,
+
+   // set deep
+   (void*) co_fail_unb,
+   (void*) co_fail_unb,
+   (void*) co_generic_getproperty,
+   (void*) co_fail_unb,
+
+   //call
+   (void*) co_fail_unb
+
+};
+
 CommOpsTable CommOpsDict[] =
 {
    NilCommOpsTable,
@@ -2582,7 +2753,8 @@ CommOpsTable CommOpsDict[] =
    ReferenceCommOpsTable,
    ClsMethodCommOpsTable,
    MethodCommOpsTable,
-   ClassCommOpsTable
+   ClassCommOpsTable,
+   UnbCommOpsTable
 };
 
 }
