@@ -112,7 +112,7 @@ void ModuleLoader::getModuleName( const String &path, String &modName )
 
    int32 dotpos = path.rfind( "." );
    int32 slashpos = path.rfind( "/" );
-   if ( dotpos == -1 )
+   if ( dotpos == -1  || dotpos < slashpos )
       dotpos = path.length();
    // legal also if slashpos < 0
 
@@ -331,7 +331,6 @@ bool ModuleLoader::scanForFile( URI &origUri, VFSProvider* vfs, t_filetype &type
       }
       else if ( vfs->readStats( origUri, fs ) )
       {
-         //type = fileType( *ext );
          type = *ptf;
          return true;
       }
@@ -399,7 +398,17 @@ Module *ModuleLoader::loadFile( const URI& uri, t_filetype type, bool scan )
       // ... if type is t_none, we may anyhow scan for a proper extension.
       if( (type == t_none || type == t_defaultSource) && ! bHadExtension )
       {
+         URI saveUri = origUri;
          bFound = scanForFile( origUri, vfs, tf, foundStats );
+         if ( ! bFound && type == t_defaultSource )
+         {
+            origUri = saveUri;
+
+            // scanforfile may
+            bFound = vfs->readStats( origUri, foundStats );
+            if ( bFound )
+               tf = t_source;
+         }
       }
       else {
          // just check if the file exists.
@@ -416,7 +425,7 @@ Module *ModuleLoader::loadFile( const URI& uri, t_filetype type, bool scan )
       while ( (! bFound) && path_elem != 0 )
       {
          String *pathp = (String *) path_elem->data();
-         origUri.pathElement().setLocation( *pathp );
+         origUri.pathElement().setFullLocation( *pathp );
          origUri.pathElement().extendLocation( oldPath  );
 
          // If we originally had an extension, we must not add it.
