@@ -23,31 +23,32 @@
 #include <falcon/string.h>
 #include <falcon/traits.h>
 #include <falcon/pcodes.h>
-#include <falcon/enginedata.h>
+#include <falcon/mt.h>
+#include <falcon/attribmap.h>
+
+#include <string.h>
 
 namespace Falcon {
 
 const uint32 Module::c_noEntry = 0xFFffFFff;
 
 Module::Module():
-   m_code( 0 ),
-   m_codeSize( 0 ),
-   m_entry( c_noEntry ),
    m_refcount(1),
-   m_lineInfo( 0 ),
+   m_language( "C" ),
    m_modVersion( 0 ),
    m_engineVersion( 0 ),
+   m_lineInfo( 0 ),
    m_loader(0),
-   m_language( "C" ),
-   m_serviceMap( &traits::t_string, &traits::t_voidp )
-{}
+   m_serviceMap( &traits::t_string(), &traits::t_voidp() ),
+   m_attributes(0)
+{
+   m_strTab = new StringTable;
+   m_bOwnStringTable = true;
+}
 
 
 Module::~Module()
 {
-   if ( m_code != 0 )
-      memFree( m_code );
-
    // The depend table points to the string table
    // so there is no need to check it.
 
@@ -56,6 +57,11 @@ Module::~Module()
    // ... the module will take care of them.
 
    delete m_lineInfo;
+
+   delete m_attributes;
+
+   if ( m_bOwnStringTable )
+      delete m_strTab;
 }
 
 DllLoader &Module::dllLoader()
@@ -66,24 +72,23 @@ DllLoader &Module::dllLoader()
    return *m_loader;
 }
 
-bool Module::addDepend( String *dep )
+void Module::addDepend( const String &name, const String &module, bool bPrivate, bool bFile )
 {
-   m_depend.pushBack( dep );
-   return true;
+   m_depend.addDependency( name, module, bPrivate, bFile );
 }
 
+void Module::addDepend( const String &name, bool bPrivate, bool bFile )
+{
+   m_depend.addDependency( name, name, bPrivate, bFile );
+}
+
+
 Symbol *Module::addGlobal( const String &name, bool exp )
 {
    if ( m_symtab.findByName( name ) != 0 )
       return 0;
 
-   String *symName = m_strTab.find( name );
-   if( symName == 0 ) {
-      symName = new String( name );
-      m_strTab.add( symName );
-   }
-
-   Symbol *sym = new Symbol( this, m_symbols.size(), symName, exp );
+   Symbol *sym = new Symbol( this, m_symbols.size(), name, exp );
    sym->setGlobal( );
    m_symbols.push( sym );
 
@@ -109,8 +114,7 @@ Symbol *Module::addSymbol()
 
 Symbol *Module::addSymbol( const String &name )
 {
-   String *n = addString( name );
-   Symbol *sym = new Symbol( this, m_symbols.size(), n, false );
+   Symbol *sym = new Symbol( this, m_symbols.size(), name, false );
    m_symbols.push( sym );
    return sym;
 }
@@ -128,9 +132,18 @@ Symbol *Module::addGlobalSymbol( Symbol *sym )
    return sym;
 }
 
+void Module::adoptStringTable( StringTable *st, bool bOwn )
+{
+   if ( m_bOwnStringTable )
+      delete m_strTab;
+   m_strTab = st;
+   m_bOwnStringTable = bOwn;
+}
+
+
 Symbol *Module::addConstant( const String &name, int64 value, bool exp )
 {
-   Symbol *sym = new Symbol( this, addString( name ) );
+   Symbol *sym = new Symbol( this, name );
    sym->exported( exp );
    sym->setConst( new VarDef( value ) );
    return addGlobalSymbol( sym );
@@ -138,7 +151,7 @@ Symbol *Module::addConstant( const String &name, int64 value, bool exp )
 
 Symbol *Module::addConstant( const String &name, numeric value, bool exp )
 {
-   Symbol *sym = new Symbol( this, addString( name ) );
+   Symbol *sym = new Symbol( this, name );
    sym->exported( exp );
    sym->setConst( new VarDef( value ) );
    return addGlobalSymbol( sym );
@@ -146,25 +159,35 @@ Symbol *Module::addConstant( const String &name, numeric value, bool exp )
 
 Symbol *Module::addConstant( const String &name, const String &value, bool exp )
 {
-   Symbol *sym = new Symbol( this, addString( name ) );
+   Symbol *sym = new Symbol( this, name );
    sym->exported( exp );
    sym->setConst( new VarDef( addString( value ) ) );
    return addGlobalSymbol( sym );
 }
 
 
-Symbol *Module::addExtFunc( const String &name, ext_func_t func, bool exp )
+Symbol *Module::addExtFunc( const String &name, ext_func_t func, void *extra, bool exp )
 {
-   Symbol *sym = addGlobalSymbol( addSymbol(name) );
-   sym->setExtFunc( new ExtFuncDef( func ) );
+   Symbol *sym = findGlobalSymbol( name );
+   if ( sym == 0 )
+   {
+      sym = addSymbol(name);
+      addGlobalSymbol( sym );
+   }
+   sym->setExtFunc( new ExtFuncDef( func, extra ) );
    sym->exported( exp );
    return sym;
 }
 
-Symbol *Module::addFunction( const String &name, uint32 offset, bool exp )
+Symbol *Module::addFunction( const String &name, byte *code, uint32 size, bool exp )
 {
-   Symbol *sym = addGlobalSymbol( addSymbol(name) );
-   sym->setFunction( new FuncDef( offset ) );
+   Symbol *sym = findGlobalSymbol( name );
+   if ( sym == 0 )
+   {
+      sym = addSymbol(name);
+      addGlobalSymbol( sym );
+   }
+   sym->setFunction( new FuncDef( code, size ) );
    sym->exported( exp );
    return sym;
 }
@@ -175,14 +198,8 @@ Symbol *Module::addClass( const String &name, Symbol *ctor_sym, bool exp )
    if ( m_symtab.findByName( name ) != 0 )
       return 0;
 
-   String *symName = m_strTab.find( name );
-   if( symName == 0 ) {
-      symName = new String( name );
-      m_strTab.add( symName );
-   }
-
-   Symbol *sym = new Symbol( this, m_symbols.size(), symName, exp );
-   sym->setClass( new ClassDef( 0, ctor_sym ) );
+   Symbol *sym = new Symbol( this, m_symbols.size(), name, exp );
+   sym->setClass( new ClassDef( ctor_sym ) );
    m_symbols.push( sym );
 
    sym->itemId( m_symtab.size() );
@@ -202,8 +219,8 @@ Symbol *Module::addSingleton( const String &name, Symbol *ctor_sym, bool exp )
    // create the class symbol (never exported)
    Symbol *clSym = addClass( clName, ctor_sym, false );
 
-   // create a singletone instance of the class.
-   Symbol *objSym = new Symbol( this, addString( name ) );
+   // create a singleton instance of the class.
+   Symbol *objSym = new Symbol( this, name );
    objSym->setInstance( clSym );
    objSym->exported( exp );
    addGlobalSymbol( objSym );
@@ -213,12 +230,19 @@ Symbol *Module::addSingleton( const String &name, Symbol *ctor_sym, bool exp )
 
 Symbol *Module::addSingleton( const String &name, ext_func_t ctor, bool exp )
 {
-   String ctor_name = name + "._init";
-   Symbol *sym = addExtFunc( ctor_name, ctor, false );
-   if ( sym == 0 )
-      return 0;
+   if( ctor != 0 )
+   {
+      String ctor_name = name + "._init";
+      Symbol *sym = addExtFunc( ctor_name, ctor, false );
+      if ( sym == 0 )
+         return 0;
 
-   return addSingleton( name, sym, exp );
+      return addSingleton( name, sym, exp );
+   }
+   else
+   {
+      return addSingleton( name, (Symbol*)0, exp );
+   }
 }
 
 
@@ -231,23 +255,23 @@ Symbol *Module::addClass( const String &name, ext_func_t ctor, bool exp )
    return addClass( name, sym, exp );
 }
 
-VarDef *Module::addClassProperty( Symbol *cls, const String &prop )
+VarDef& Module::addClassProperty( Symbol *cls, const String &prop )
 {
    ClassDef *cd = cls->getClassDef();
    VarDef *vd = new VarDef();
    cd->addProperty( addString( prop ), vd );
-   return vd;
+   return *vd;
 }
 
-VarDef *Module::addClassMethod( Symbol *cls, const String &prop, Symbol *method )
+VarDef& Module::addClassMethod( Symbol *cls, const String &prop, Symbol *method )
 {
    ClassDef *cd = cls->getClassDef();
    VarDef *vd = new VarDef( method );
    cd->addProperty( addString( prop ), vd );
-   return vd;
+   return *vd;
 }
 
-VarDef *Module::addClassMethod( Symbol *cls, const String &prop, ext_func_t method_func )
+VarDef& Module::addClassMethod( Symbol *cls, const String &prop, ext_func_t method_func )
 {
    String name = cls->name() + "." + prop;
    Symbol *method = addExtFunc( name, method_func, false );
@@ -256,38 +280,38 @@ VarDef *Module::addClassMethod( Symbol *cls, const String &prop, ext_func_t meth
 
 String *Module::addCString( const char *pos, uint32 size )
 {
-   String *ret = m_strTab.find( pos );
+   String *ret = stringTable().find( pos );
    if ( ret == 0 ) {
       char *mem = (char *)memAlloc(size+1);
       memcpy( mem, pos, size );
       mem[size] = 0;
       ret = new String();
       ret->adopt( mem, size, size + 1 );
-      m_strTab.add( ret );
+      stringTable().add( ret );
    }
    return ret;
 }
 
 String *Module::addString( const String &st )
 {
-   String *ret = m_strTab.find( st );
+   String *ret = stringTable().find( st );
    if ( ret == 0 ) {
       ret = new String( st );
       ret->bufferize();
       ret->exported( st.exported() );
-		m_strTab.add( ret );
+      stringTable().add( ret );
    }
    return ret;
 }
 
 String *Module::addString( const String &st, bool exported )
 {
-   String *ret = m_strTab.find( st );
+   String *ret = stringTable().find( st );
    if ( ret == 0 ) {
       ret = new String( st );
       ret->exported( exported );
       ret->bufferize();
-		m_strTab.add( ret );
+      stringTable().add( ret );
    }
    else {
       ret->exported( exported );
@@ -296,6 +320,17 @@ String *Module::addString( const String &st, bool exported )
    return ret;
 }
 
+uint32 Module::addStringID( const String &st, bool exported )
+{
+   uint32 ret = stringTable().size();
+   String* s = new String( st );
+   s->exported( exported );
+   s->bufferize();
+   stringTable().add( s );
+
+   return ret;
+}
+
 bool Module::save( Stream *out, bool skipCode ) const
 {
    // save header informations:
@@ -316,7 +351,7 @@ bool Module::save( Stream *out, bool skipCode ) const
    // serialize the module tables.
    //NOTE: all the module tables are saved 32 bit alinged.
 
-   if ( ! m_strTab.save( out ) )
+   if ( ! stringTable().save( out ) )
       return false;
 
    if ( ! m_symbols.save( out ) )
@@ -328,6 +363,20 @@ bool Module::save( Stream *out, bool skipCode ) const
    if ( ! m_depend.save( out ) )
       return false;
 
+
+   if( m_attributes != 0 )
+   {
+      iver = endianInt32(1);
+      out->write( &iver, sizeof( iver ) );
+
+      if ( ! m_attributes->save( this, out ) )
+         return false;
+   }
+   else {
+      iver = endianInt32(0);
+      out->write( &iver, sizeof( iver ) );
+   }
+
    if ( m_lineInfo != 0 )
    {
       int32 infoInd = endianInt32( 1 );
@@ -341,22 +390,6 @@ bool Module::save( Stream *out, bool skipCode ) const
       out->write( &infoInd, sizeof( infoInd ) );
    }
 
-   uint32 entry = endianInt32( m_entry );
-   out->write( &entry, sizeof( entry ) );
-
-   // save the code
-   // later on, the system may rewrite the code size field.
-   // this may be useful to use the file that is being created to compile
-   // on the fly the bytecode.
-   if ( ! skipCode )
-   {
-      uint32 cs = endianInt32( m_codeSize );
-      out->write( &cs, sizeof( cs ) );
-
-      if ( m_codeSize !=  0 )
-         out->write( m_code, m_codeSize );
-   }
-
    return out->good();
 }
 
@@ -390,7 +423,7 @@ bool Module::load( Stream *is, bool skipHeader )
    /*TODO: see if there is a localized table around and use that instead.
       If so, skip our internal strtable.
    */
-   if ( ! m_strTab.load( is ) )
+   if ( ! stringTable().load( is ) )
       return false;
 
    if ( ! m_symbols.load( this, is ) )
@@ -402,6 +435,14 @@ bool Module::load( Stream *is, bool skipHeader )
    if ( ! m_depend.load( this, is ) )
       return false;
 
+   is->read( &ver, sizeof( ver ) );
+   if( ver != 0 )
+   {
+      m_attributes = new AttribMap;
+      if ( ! m_attributes->load( this, is ) )
+         return false;
+   }
+
    // load lineinfo indicator.
    int32 infoInd;
    is->read( &infoInd, sizeof( infoInd ) );
@@ -415,33 +456,9 @@ bool Module::load( Stream *is, bool skipHeader )
    else
       m_lineInfo = 0;
 
-   // Read the entry point
-   uint32 cs;
-   is->read( &cs, sizeof( cs ) );
-   m_entry = endianInt32( cs );
-
-   // read the code
-   is->read( &cs, sizeof( cs ) );
-   m_codeSize = endianInt32( cs );
-   if ( m_codeSize !=  0 ) {
-      m_code = (byte *) memAlloc( m_codeSize );
-      is->read(  m_code, m_codeSize );
-   }
-
    return is->good();
 }
 
-void Module::addMain()
-{
-   if( code() != 0 && entry() != 0xFFffFFff )
-   {
-      Symbol *sym = findGlobalSymbol( "__main__" );
-      if ( sym == 0 ) {
-         sym = addFunction( "__main__", entry() );
-         sym->exported(false);
-      }
-   }
-}
 
 uint32 Module::getLineAt( uint32 pc ) const
 {
@@ -532,17 +549,18 @@ DllLoader *Module::detachLoader()
 }
 
 
-void Module::incref()
+void Module::incref() const
 {
-   Engine::atomicInc( m_refcount );
+   atomicInc( m_refcount );
 }
 
-void Module::decref()
+void Module::decref() const
 {
-   if( Engine::atomicDec( m_refcount ) <= 0 )
+   if( atomicDec( m_refcount ) <= 0 )
    {
-      DllLoader *loader = detachLoader();
-      delete this;
+      Module *deconst = const_cast<Module *>(this);
+      DllLoader *loader = deconst->detachLoader();
+      delete deconst;
       delete loader;
    }
 }
@@ -552,7 +570,60 @@ bool Module::saveTableTemplate( Stream *stream, const String &encoding ) const
    stream->writeString( "<?xml version=\"1.0\" encoding=\"" );
    stream->writeString( encoding );
    stream->writeString( "\"?>\n" );
-   return m_strTab.saveTemplate( stream, name(), language() );
+   return stringTable().saveTemplate( stream, name(), language() );
+}
+
+
+String Module::absoluteName( const String &module_name, const String &parent_name )
+{
+   if ( module_name.getCharAt(0) == '.' )
+   {
+      // notation .name
+      if ( parent_name.size() == 0 )
+         return module_name.subString( 1 );
+      else {
+         // remove last part of parent name
+         uint32 posDot = parent_name.rfind( "." );
+         // are there no dot? -- we're at root elements
+         if ( posDot == String::npos )
+            return module_name.subString( 1 );
+         else
+            return parent_name.subString( 0, posDot ) + module_name; // "." is included.
+      }
+   }
+   else if ( module_name.find( "self." ) == 0 )
+   {
+      if ( parent_name.size() == 0 )
+         return module_name.subString( 5 );
+      else
+         return parent_name + "." + module_name.subString( 5 );
+   }
+   else
+      return module_name;
+}
+
+
+void Module::addAttribute( const String &name, VarDef* vd )
+{
+   if( m_attributes == 0 )
+      m_attributes = new AttribMap;
+
+   m_attributes->insertAttrib( name, vd );
+}
+
+
+void Module::rollBackSymbols( uint32 nsize )
+{
+   uint32 size = symbols().size();
+
+   for (uint32 pos =  nsize; pos < size; pos ++ )
+   {
+      Symbol *sym = symbols().symbolAt( pos );
+      symbolTable().remove( sym->name() );
+      delete sym;
+   }
+
+   symbols().resize(nsize);
 }
 
 }
