@@ -123,7 +123,7 @@ private:
 public:
    virtual const String encoding() const { return "gbk"; }
 
-   virtual FalconData *clone() const
+   virtual TranscoderGBK *clone() const
    {
       return new TranscoderGBK( *this );
    }
@@ -207,7 +207,10 @@ class TranscoderISO_CP: public Transcoder
 protected:
    TranscoderISO_CP( Stream *s, bool bOwn=false ):
        Transcoder( s, bOwn )
-       {}
+       {
+          m_nMinUntl = 0x0;
+          m_nMaxUntl = 0x9F;
+       }
 
     TranscoderISO_CP( const TranscoderISO_CP &other );
 
@@ -215,12 +218,43 @@ protected:
        uint32 m_dirTabSize;
        CP_ISO_UINT_TABLE *m_reverseTable;
        uint32 m_revTabSize;
+       uint32 m_nMinUntl;
+       uint32 m_nMaxUntl;
 
 public:
    virtual bool get( uint32 &chr );
    virtual bool put( uint32 chr );
 };
 
+
+class TranscoderOEM: public TranscoderISO_CP
+{
+protected:
+   TranscoderOEM( Stream *s, bool bOwn=false ):
+       TranscoderISO_CP( s, bOwn )
+       {
+          m_nMinUntl = 0x00;
+          m_nMaxUntl = 0x7F;
+       }
+
+    TranscoderOEM( const TranscoderOEM &other );
+public:
+   virtual bool get( uint32 &chr );
+};
+
+
+class TranscoderIBM437:public TranscoderOEM
+{
+public:
+   TranscoderIBM437( const TranscoderIBM437 &other ):
+     TranscoderOEM( other )
+     {}
+
+     TranscoderIBM437( Stream *s, bool bOwn=false );
+     virtual const String encoding() const { return "IBM437"; }
+     virtual TranscoderIBM437 *clone() const;
+};
+
 class TranscoderCP1252:public TranscoderISO_CP
 {
 public:
@@ -230,7 +264,7 @@ public:
 
      TranscoderCP1252( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "cp1252"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderCP1252 *clone() const;
 };
 
 /** Latin-1 (ISO8859_1) transcoder. */
@@ -243,7 +277,7 @@ public:
 
      TranscoderISO8859_1( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-1"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_1 *clone() const;
 };
 
 class TranscoderISO8859_2: public TranscoderISO_CP
@@ -255,7 +289,7 @@ public:
 
      TranscoderISO8859_2( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-2"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_2 *clone() const;
 };
 
 class TranscoderISO8859_3: public TranscoderISO_CP
@@ -267,7 +301,7 @@ public:
 
      TranscoderISO8859_3( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-3"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_3 *clone() const;
 };
 
 
@@ -280,7 +314,7 @@ public:
 
      TranscoderISO8859_4( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-4"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_4 *clone() const;
 };
 
 class TranscoderISO8859_5: public TranscoderISO_CP
@@ -292,7 +326,7 @@ public:
 
      TranscoderISO8859_5( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-5"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_5 *clone() const;
 };
 
 class TranscoderISO8859_6: public TranscoderISO_CP
@@ -304,7 +338,7 @@ public:
 
      TranscoderISO8859_6( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-6"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_6 *clone() const;
 };
 
 class TranscoderISO8859_7: public TranscoderISO_CP
@@ -316,7 +350,7 @@ public:
 
      TranscoderISO8859_7( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-7"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_7 *clone() const;
 };
 
 class TranscoderISO8859_8: public TranscoderISO_CP
@@ -328,7 +362,7 @@ public:
 
      TranscoderISO8859_8( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-8"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_8 *clone() const;
 };
 
 class TranscoderISO8859_9: public TranscoderISO_CP
@@ -340,7 +374,7 @@ public:
 
      TranscoderISO8859_9( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-9"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_9 *clone() const;
 };
 
 class TranscoderISO8859_10: public TranscoderISO_CP
@@ -352,7 +386,7 @@ public:
 
      TranscoderISO8859_10( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-10"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_10 *clone() const;
 };
 
 class TranscoderISO8859_11: public TranscoderISO_CP
@@ -364,7 +398,7 @@ public:
 
      TranscoderISO8859_11( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-11"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_11 *clone() const;
 };
 
 class TranscoderISO8859_13: public TranscoderISO_CP
@@ -376,7 +410,7 @@ public:
 
      TranscoderISO8859_13( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-13"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_13 *clone() const;
 };
 
 class TranscoderISO8859_14: public TranscoderISO_CP
@@ -388,7 +422,7 @@ public:
 
      TranscoderISO8859_14( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-14"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_14 *clone() const;
 };
 
 class TranscoderISO8859_15: public TranscoderISO_CP
@@ -400,7 +434,7 @@ public:
 
      TranscoderISO8859_15( Stream *s, bool bOwn=false );
      virtual const String encoding() const { return "iso8859-15"; }
-     virtual FalconData *clone() const;
+     virtual TranscoderISO8859_15 *clone() const;
 };
 
 //=============================================================================
@@ -569,7 +603,7 @@ bool TranscoderByte::put( uint32 chr )
    return ( m_stream->write( &b, 1 ) == 1 );
 }
 
-FalconData *TranscoderByte::clone() const
+TranscoderByte *TranscoderByte::clone() const
 {
    return new TranscoderByte( *this );
 }
@@ -620,7 +654,7 @@ bool TranscoderEOL::put( uint32 chr )
    return m_stream->put( chr );
 }
 
-FalconData *TranscoderEOL::clone() const
+TranscoderEOL *TranscoderEOL::clone() const
 {
    return new TranscoderEOL( *this );
 }
@@ -741,7 +775,7 @@ bool TranscoderUTF8::put( uint32 chr )
    return ( m_stream->write( res, resCount ) == (int32) resCount);
 }
 
-FalconData *TranscoderUTF8::clone() const
+TranscoderUTF8 *TranscoderUTF8::clone() const
 {
    return new TranscoderUTF8( *this );
 }
@@ -939,11 +973,13 @@ bool TranscoderUTF16::put( uint32 chr )
    return true;
 }
 
-FalconData *TranscoderUTF16::clone() const
+TranscoderUTF16 *TranscoderUTF16::clone() const
 {
    return new TranscoderUTF16( *this );
 }
 
+
+
 //=========================================
 // TABLES
 //
@@ -952,7 +988,9 @@ TranscoderISO_CP::TranscoderISO_CP( const TranscoderISO_CP &other ):
    m_directTable( other.m_directTable ),
    m_dirTabSize( other.m_dirTabSize ),
    m_reverseTable( other.m_reverseTable ),
-   m_revTabSize( other.m_revTabSize )
+   m_revTabSize( other.m_revTabSize ),
+   m_nMinUntl( m_nMinUntl ),
+   m_nMaxUntl( m_nMaxUntl )
 {}
 
 bool TranscoderISO_CP::get( uint32 &chr )
@@ -993,7 +1031,7 @@ bool TranscoderISO_CP::put( uint32 chr )
 {
    m_parseStatus = true;
    byte out;
-   if ( chr < 0xA0 )
+   if ( chr >= m_nMinUntl && chr <= m_nMaxUntl )
    {
       out = (byte) chr;
    }
@@ -1046,7 +1084,6 @@ bool TranscoderISO_CP::put( uint32 chr )
    return (m_stream->write( &out, 1 ) == 1);
 }
 
-
 TranscoderCP1252::TranscoderCP1252( Stream *s, bool bOwn ):
    TranscoderISO_CP( s, bOwn )
 {
@@ -1056,7 +1093,7 @@ TranscoderCP1252::TranscoderCP1252( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_cp1252 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderCP1252::clone() const
+TranscoderCP1252 *TranscoderCP1252::clone() const
 {
    return new TranscoderCP1252( *this );
 }
@@ -1070,7 +1107,7 @@ TranscoderISO8859_1::TranscoderISO8859_1( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_1 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_1::clone() const
+TranscoderISO8859_1 *TranscoderISO8859_1::clone() const
 {
    return new TranscoderISO8859_1( *this );
 }
@@ -1084,7 +1121,7 @@ TranscoderISO8859_2::TranscoderISO8859_2( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_2 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_2::clone() const
+TranscoderISO8859_2 *TranscoderISO8859_2::clone() const
 {
    return new TranscoderISO8859_2( *this );
 }
@@ -1098,7 +1135,7 @@ TranscoderISO8859_3::TranscoderISO8859_3( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_3 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_3::clone() const
+TranscoderISO8859_3 *TranscoderISO8859_3::clone() const
 {
    return new TranscoderISO8859_3( *this );
 }
@@ -1113,7 +1150,7 @@ TranscoderISO8859_4::TranscoderISO8859_4( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_4 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_4::clone() const
+TranscoderISO8859_4 *TranscoderISO8859_4::clone() const
 {
    return new TranscoderISO8859_4( *this );
 }
@@ -1128,7 +1165,7 @@ TranscoderISO8859_5::TranscoderISO8859_5( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_5 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_5::clone() const
+TranscoderISO8859_5 *TranscoderISO8859_5::clone() const
 {
    return new TranscoderISO8859_5( *this );
 }
@@ -1143,7 +1180,7 @@ TranscoderISO8859_6::TranscoderISO8859_6( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_6 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_6::clone() const
+TranscoderISO8859_6 *TranscoderISO8859_6::clone() const
 {
    return new TranscoderISO8859_6( *this );
 }
@@ -1157,7 +1194,7 @@ TranscoderISO8859_7::TranscoderISO8859_7( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_7 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_7::clone() const
+TranscoderISO8859_7 *TranscoderISO8859_7::clone() const
 {
    return new TranscoderISO8859_7( *this );
 }
@@ -1171,7 +1208,7 @@ TranscoderISO8859_8::TranscoderISO8859_8( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_8 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_8::clone() const
+TranscoderISO8859_8 *TranscoderISO8859_8::clone() const
 {
    return new TranscoderISO8859_8( *this );
 }
@@ -1185,7 +1222,7 @@ TranscoderISO8859_9::TranscoderISO8859_9( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_9 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_9::clone() const
+TranscoderISO8859_9 *TranscoderISO8859_9::clone() const
 {
    return new TranscoderISO8859_9( *this );
 }
@@ -1200,7 +1237,7 @@ TranscoderISO8859_10::TranscoderISO8859_10( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_10 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_10::clone() const
+TranscoderISO8859_10 *TranscoderISO8859_10::clone() const
 {
    return new TranscoderISO8859_10( *this );
 }
@@ -1214,7 +1251,7 @@ TranscoderISO8859_11::TranscoderISO8859_11( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_11 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_11::clone() const
+TranscoderISO8859_11 *TranscoderISO8859_11::clone() const
 {
    return new TranscoderISO8859_11( *this );
 }
@@ -1228,7 +1265,7 @@ TranscoderISO8859_13::TranscoderISO8859_13( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_13 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_13::clone() const
+TranscoderISO8859_13 *TranscoderISO8859_13::clone() const
 {
    return new TranscoderISO8859_13( *this );
 }
@@ -1242,7 +1279,7 @@ TranscoderISO8859_14::TranscoderISO8859_14( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_14 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_14::clone() const
+TranscoderISO8859_14 *TranscoderISO8859_14::clone() const
 {
    return new TranscoderISO8859_14( *this );
 }
@@ -1256,11 +1293,67 @@ TranscoderISO8859_15::TranscoderISO8859_15( Stream *s, bool bOwn ):
    m_revTabSize = sizeof( s_rtable_iso8859_15 ) / sizeof( CP_ISO_UINT_TABLE );
 }
 
-FalconData *TranscoderISO8859_15::clone() const
+TranscoderISO8859_15 *TranscoderISO8859_15::clone() const
 {
    return new TranscoderISO8859_15( *this );
 }
 
+
+//=========================================
+// OEM tables (covering 0x0 to 0x20  and 0x80 to 0xff ranges)
+//
+
+TranscoderOEM::TranscoderOEM( const TranscoderOEM &other ):
+   TranscoderISO_CP( other )
+{}
+
+bool TranscoderOEM::get( uint32 &chr )
+{
+   m_parseStatus = true;
+
+   if( popBuffer( chr ) )
+      return true;
+
+   // converting the character into an unicode.
+   byte in;
+   if ( m_stream->read( &in, 1 ) != 1 )
+      return false;
+
+   if ( in >= m_nMinUntl && in <= m_nMaxUntl )
+   {
+      chr = (uint32) in;
+      return true;
+   }
+
+   if ( in >= 0xFF )
+   {
+      chr = (uint32) '?';
+      m_parseStatus = false;
+      return true;
+   }
+
+   chr = chr < m_nMinUntl ? 
+      (uint32) m_directTable[ in ] :
+      (uint32) m_directTable[ in + m_nMinUntl - m_nMaxUntl-1 ];
+
+   return true;
+}
+
+TranscoderIBM437::TranscoderIBM437( Stream *s, bool bOwn ):
+   TranscoderOEM( s, bOwn )
+{
+   m_directTable = s_table_IBM437;
+   m_dirTabSize = sizeof( s_table_IBM437 ) / sizeof( uint16 );
+   m_reverseTable = s_rtable_IBM437;
+   m_revTabSize = sizeof( s_rtable_IBM437 ) / sizeof( CP_ISO_UINT_TABLE );
+}
+
+TranscoderIBM437 *TranscoderIBM437::clone() const
+{
+   return new TranscoderIBM437( *this );
+}
+
+
 //==================================================================
 // Utilities
 
@@ -1285,6 +1378,9 @@ Transcoder *TranscoderFactory( const String &encoding, Stream *stream, bool own
    if ( encoding == "cp1252" )
       return new TranscoderCP1252( stream, own );
 
+   if ( encoding == "IBM437" )
+      return new TranscoderIBM437( stream, own );
+
    if ( encoding == "iso8859-1" )
       return new TranscoderISO8859_1( stream, own );
 
@@ -1376,7 +1472,7 @@ bool GetSystemEncodingWin(String &encoding)
    switch (ConsoleInputCP)
    {
    case 37: encoding = "IBM037";	return false;   // IBM EBCDIC US-Canada
-   case 437: encoding = "IBM437";	return false;   // OEM United States
+   case 437: encoding = "IBM437";	return true;   // OEM United States
    case 500: encoding = "IBM500";	return false; 	// IBM EBCDIC International
    case 708: encoding = "ASMO-708";	return false; 	// Arabic (ASMO 708)
    case 709: encoding = "";	return false; 	// Arabic (ASMO-449+, BCON V4)
@@ -1597,13 +1693,17 @@ bool GetSystemEncoding( String &encoding )
       }
       else {
          // utf variant?
-         pos = encoding.find( "UTF-" );
+         pos = encoding.find( "UTF" );
          if ( pos == csh::npos )
-            pos = encoding.find( "utf-" );
-
-         if ( pos != csh::npos && pos < encoding.length() - 4 )
+            pos = encoding.find( "utf" );
+         
+         uint32 utf_prefix_len=3; 
+         if(pos != csh::npos && encoding.getCharAt(utf_prefix_len) == '-' )
+           utf_prefix_len++;;
+         
+         if ( pos != csh::npos && pos < encoding.length() - utf_prefix_len )
          {
-            encoding = encoding.subString( pos + 4 );
+            encoding = encoding.subString( pos + utf_prefix_len );
             if ( encoding == "8" || encoding == "16" || encoding == "16LE" || encoding == "16BE" )
             {
                encoding = "utf-" + encoding;
