@@ -90,6 +90,10 @@ Item *VMachine::getOpcodeParam( register uint32 bc_pos )
          m_imm[bc_pos].setNil();
       return m_imm + bc_pos;
 
+      case P_PARAM_UNB:
+         m_imm[bc_pos].setUnbound();
+      return m_imm + bc_pos;
+
       case P_PARAM_GLOBID:
       {
          register int32 id = *reinterpret_cast< int32 * >( m_currentContext->code() + m_currentContext->pc_next() );
@@ -99,14 +103,12 @@ Item *VMachine::getOpcodeParam( register uint32 bc_pos )
       break;
 
       case P_PARAM_LOCID:
-         ret = &stackItem( stackBase() +
-               *reinterpret_cast< int32 * >( m_currentContext->code() + m_currentContext->pc_next() ) );
+         ret = local( *reinterpret_cast< int32 * >( m_currentContext->code() + m_currentContext->pc_next() ) );
          m_currentContext->pc_next()+=sizeof(int32);
       return ret;
 
       case P_PARAM_PARID:
-         ret = &stackItem( stackBase() - paramCount() - VM_FRAME_SPACE +
-               *reinterpret_cast< int32 * >( m_currentContext->code() + m_currentContext->pc_next() ) );
+         ret = param( *reinterpret_cast< int32 * >( m_currentContext->code() + m_currentContext->pc_next() ) );
          m_currentContext->pc_next()+=sizeof(int32);
       return ret;
 
@@ -140,7 +142,7 @@ void VMachine::run()
    // declare this as the running machine
    setCurrent();
 
-   while( true )
+   while( ! m_break )
    {
       try {
          // move m_currentContext->pc_next() to the end of instruction and beginning of parameters.
@@ -192,14 +194,8 @@ void VMachine::run()
          }
 
          // Jump to next isntruction.
-
          m_currentContext->pc() = m_currentContext->pc_next();
       }
-      catch( VMEventReturn & )
-      {
-         //m_currentContext->pc() = m_currentContext->pc_next();
-         return;
-      }
       // catches explicitly raised items.
       catch( Item& raised )
       {
@@ -213,6 +209,8 @@ void VMachine::run()
          handleRaisedError( err );
       }
    } // end while -- VM LOOP
+
+   m_break = false;
 }
 
 /****************************************************
@@ -241,8 +239,8 @@ void opcodeHandler_PSHN( register VMachine *vm )
 // 3
 void opcodeHandler_RET( register VMachine *vm )
 {
-   vm->callReturn();
    vm->retnil();
+   vm->callReturn();
 }
 
 // 4
@@ -303,11 +301,8 @@ void opcodeHandler_GENA( register VMachine *vm )
    // copy the m-topmost items in the stack into the array
    if( size > 0 )
    {
-      Item *data = array->items().elements();
-      int32 base = vm->stack().length() - size;
-      memcpy( data, &vm->stack()[ base ], sizeof(Item)*size );
-      array->length( size );
-      vm->stack().resize( base );
+      array->items().copyOnto( vm->stack(), vm->stack().length() - size,  size );
+      vm->currentFrame()->pop( size );
    }
 }
 
@@ -363,7 +358,7 @@ void opcodeHandler_PSHR( register VMachine *vm )
 // 0E
 void opcodeHandler_POP( register VMachine *vm )
 {
-   if ( vm->stack().length() == vm->stackBase() ) {
+   if ( vm->stack().length() == 0 ) {
       vm->raiseHardError( e_stackuf, "POP", __LINE__ );
       return;
    }
@@ -799,8 +794,6 @@ void opcodeHandler_GENR( register VMachine *vm )
 
 
    int64 secondOp = operand2->forceIntegerEx();
-   if( operand2->isOob() && secondOp > 0 )
-      secondOp++;
 
    int64 step;
    if ( operand3->isNil() )
@@ -812,8 +805,12 @@ void opcodeHandler_GENR( register VMachine *vm )
       step = operand3->forceIntegerEx();
    }
 
+
    int64 firstOp = operand1->forceIntegerEx();
 
+   if( operand2->isOob() && firstOp <= secondOp )
+      secondOp++;
+
    vm->regA().setRange( new CoreRange(
       firstOp,
       secondOp,
@@ -1443,7 +1440,7 @@ void opcodeHandler_TRAV( register VMachine *vm )
       if( seq == 0 )
       {
          // is it a callable entity?
-         if ( source->asObjectSafe()->hasProperty("call__"))
+         if ( source->asObjectSafe()->hasProperty( OVERRIDE_OP_CALL ))
          {
             seq = new GeneratorSeq( vm, *source );
             GarbagePointer* ptr = new GarbagePointer( seq );
@@ -1513,7 +1510,7 @@ void opcodeHandler_TRAV( register VMachine *vm )
    Item iterItem;
    Iterator* current = new Iterator( seq );
    iterItem.setGCPointer( current );
-   vm->pushParameter( iterItem );
+   vm->pushParam( iterItem );
 
    // the source is safe through back-marking of the iterator on its sequence,
    // and of the sequence on its owner.
@@ -1610,7 +1607,7 @@ void opcodeHandler_CLOS( register VMachine *vm )
 
    if( size > 0 )
    {
-      if ( size + vm->stackBase() > vm->stack().length() )
+      if ( size > vm->stack().length() )
       {
          throw
             new CodeError( ErrorParam( e_stackuf ).origin( e_orig_vm ).extra("CLOS") );
@@ -1796,17 +1793,17 @@ void opcodeHandler_STEX( register VMachine *vm )
 
       case VMachine::return_error_parse_fmt:
          throw
-            new TypeError( ErrorParam( e_fmt_convert ).origin( e_orig_vm ).extra( "STEX" ) );
+            new TypeError( ErrorParam( e_fmt_convert ).origin( e_orig_vm ).extra( "STEX-format" ) );
       break;
 
       case VMachine::return_error_string:
          throw
-            new ParamError( ErrorParam( e_param_strexp_code ).origin( e_orig_vm ).extra( "STEX" ) );
+            new ParamError( ErrorParam( e_param_strexp_code ).origin( e_orig_vm ).extra( "STEX-string" ) );
       break;
 
       case VMachine::return_error_parse:
          throw
-               new ParamError( ErrorParam( e_param_indir_code ).origin( e_orig_vm ).extra( "STEX" ) );
+               new ParamError( ErrorParam( e_param_indir_code ).origin( e_orig_vm ).extra( "STEX-parse" ) );
       break;
 
       default:  // warning no-op
@@ -1953,6 +1950,15 @@ void opcodeHandler_TRDN( register VMachine *vm )
    }
 }
 
+// 0x70
+void opcodeHandler_EXEQ( register VMachine *vm )
+{
+   Item *operand1 =  vm->getOpcodeParam( 1 );
+   Item *operand2 =  vm->getOpcodeParam( 2 );
+   vm->regA().setBoolean( operand1->exactlyEqual(*operand2) );
+}
+   
+
 }
 
 /* end of vm_run.cpp */
