@@ -194,9 +194,7 @@ public:
    bool operator <=( const TimeStamp &ts ) const { return this->compare( ts ) <= 0; }
    bool operator >=( const TimeStamp &ts ) const { return this->compare( ts ) >= 0; }
 
-   friend void Sys::Time::timestampFromSystemTime( const SystemTime &sys_time, TimeStamp &ts );
-
-   virtual FalconData *clone() const;
+   virtual TimeStamp *clone() const;
 };
 
 inline TimeStamp operator + ( const TimeStamp &ts1, const TimeStamp &ts2 )
