@@ -72,7 +72,7 @@ namespace Ext {
       default system encoding that Falcon is able to detect. Use one of the
       encoding names known by the Transcoder class.
 
-   @prop launchOnLink If true, the __main__ function (that is, the entry point)
+   @prop launchAtLink If true, the __main__ function (that is, the entry point)
          of the loaded modules is executed before returning it. This allows
          the modules to initalize themselves and set their global variables.
          Notice that this step may be autonomusly performed also by the loader
@@ -110,7 +110,7 @@ namespace Ext {
    @param dt Directive to be set.
    @param value Value to be given to the directive.
    @return On success, a @a Module instance that contains the loaded module.
-   @raise SyntaxError if the module contains logical srror.
+   @raise SyntaxError if the module contains logical error.
    @raise IoError if the input data is a file stream and there have been a read failure.
 
    Sets a directive as if the scripts that will be loaded by this compiler defined it
@@ -181,14 +181,10 @@ FALCON_FUNC BaseCompiler_addFalconPath( ::Falcon::VMachine *vm )
       end
    @endcode
 
+   @note If @b path is not provided, defaults to "."
+   (script current working directory).
 */
 
-/*#
-   @init Compiler
-   @brief Initializes the compiler with a default path.
-   If @b path is not provided, defaults to "." (script current working directory).
-
-*/
 FALCON_FUNC Compiler_init( ::Falcon::VMachine *vm )
 {
    Item *i_path = vm->param( 0 );
@@ -263,7 +259,7 @@ void internal_link( ::Falcon::VMachine *vm, Module *mod, CompilerIface *iface )
    @param modName A logical unique that will be given to the module after compilation.
    @param data The data to compile. It may be a string or a stream valid for input.
    @return On success, a @a Module instance that contains the compiled module.
-   @raise SyntaxError if the module contains logical srror.
+   @raise SyntaxError if the module contains logical error.
    @raise IoError if the input data is a file stream and there have been a read failure.
 
    Tries to compile the module in the @b data parameter. On failure, a SyntaxError
@@ -344,7 +340,7 @@ FALCON_FUNC Compiler_compile( ::Falcon::VMachine *vm )
    @brief Loads a module given its logical name.
    @param modName The logical name of the module to be loaded.
    @return On success, a @a Module instance that contains the loaded module.
-   @raise SyntaxError if the module contains logical srror.
+   @raise SyntaxError if the module contains logical error.
    @raise IoError if the input data is a file stream and there have been a read failure.
 
    Tries to load a logically named module scanning for suitable sources, pre-compiled
@@ -396,7 +392,7 @@ FALCON_FUNC Compiler_loadByName( ::Falcon::VMachine *vm )
    @param modPath Relative or absolute path to a loadable Falcon module or source.
    @optparam alias Alias under which the module should be loaded.
    @return On success, a @a Module instance that contains the loaded module.
-   @raise SyntaxError if the module contains logical srror.
+   @raise SyntaxError if the module contains logical error.
    @raise IoError if the input data is a file stream and there have been a read failure.
 
    Loads the given file, trying to perform compilation or loading of the relevant
@@ -579,12 +575,6 @@ FALCON_FUNC Compiler_loadFile( ::Falcon::VMachine *vm )
     @prop result Item containing last evaluation result.
 */
 
-/*# @init ICompiler
-    @brief Initializes the interactive compiler, eventually setting a path for
-           the module loader.
-
-    If @b path is not given, the default system path is used.
-*/
 
 FALCON_FUNC ICompiler_init( ::Falcon::VMachine *vm )
 {
@@ -689,7 +679,7 @@ FALCON_FUNC ICompiler_compileAll( ::Falcon::VMachine *vm )
    throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S" ) );
 }
 
-/*#
+/*
    @method reset ICompiler
    @brief Resets the compiler.
 
