@@ -1,6 +1,6 @@
 /*
    FALCON - The Falcon Programming Language.
-   FILE: socket.cpp
+   FILE: confparser.cpp
 
    The configuration parser module.
    -------------------------------------------------------------------
@@ -8,7 +8,7 @@
    Begin: 2006-05-09 15:50
 
    -------------------------------------------------------------------
-   (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
+   (C) Copyright 2006: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
 */
@@ -22,6 +22,13 @@
 #include "confparser_st.h"
 
 #include "version.h"
+
+#include <falcon/srv/confparser_srv.h>
+
+//TODO: Use new dynamic allocation model
+static Falcon::ConfigFileService s_config_srv;
+
+
 /*#
    @module feather_configparser Configuration Parser
    @brief Advanced configuration file parser (with sections and key categorization support).
@@ -36,7 +43,7 @@
    original INI file, so that it stays familiar for the user after a modify that
    has been caused by the Falcon module.
 
-   @section Ini file format
+   @section confparser_ini_fformat Ini file format
 
    The ConfParser module parses INI files in the following format:
    @code
@@ -66,21 +73,21 @@
    also after a value, if a value itself contains one of those characters it
    should be enclosed by quotes, or part of the value will be considered a comment.
 
-   In example:
+   For example:
 
    @code
    Key: "A complex value containing\nescapes # and comment" ; real comment
    @endcode
 
-   @subsection Multiple values
+   @subsection confparser_multiple_values Multiple values
 
    Although it would be possible to put arbitrary lists into strings to save
    them on configuration files, and expand them in the program when reading them
    back, it is possible to store array of values in configuration files by
    declaring multiple times the same key.
 
+   For example:
    @code
-   In example:
    Key = value1
    Key = value2
    Key = value3
@@ -89,7 +96,7 @@
    This will result in the three values to be returned in an array when the
    value of "Key" is asked.
 
-   @subsection Key categories
+   @subsection confparser_key_cat Key categories
 
    Keys can be categorized; tree-like or even recursive key groups can be
    represented with dots separating the key hierarchy elements.
@@ -119,12 +126,11 @@
    or if the parameter has nil value, then the methods will operate on the main
    section.
 */
-FALCON_MODULE_DECL( const Falcon::EngineData &data )
+FALCON_MODULE_DECL
 {
    #define FALCON_DECLARE_MODULE self
 
    // setup DLL engine common data
-   data.set();
 
    Falcon::Module *self = new Falcon::Module();
    self->name( "confparser" );
@@ -138,27 +144,47 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
 
 
    Falcon::Symbol *c_cparser = self->addClass( "ConfParser", Falcon::Ext::ConfParser_init );
-   self->addClassMethod( c_cparser, "read", Falcon::Ext::ConfParser_read );
-   self->addClassMethod( c_cparser, "write", Falcon::Ext::ConfParser_write );
-   self->addClassMethod( c_cparser, "get", Falcon::Ext::ConfParser_get );
-   self->addClassMethod( c_cparser, "getOne", Falcon::Ext::ConfParser_getOne );
-   self->addClassMethod( c_cparser, "getMultiple", Falcon::Ext::ConfParser_getMultiple );
+   //
+
+   self->addClassMethod( c_cparser, "read", Falcon::Ext::ConfParser_read ).asSymbol()->
+      addParam("stream");
+   self->addClassMethod( c_cparser, "write", Falcon::Ext::ConfParser_write ).asSymbol()->
+      addParam("stream");
+   self->addClassMethod( c_cparser, "get", Falcon::Ext::ConfParser_get ).asSymbol()->
+      addParam("key")->addParam("section");
+   self->addClassMethod( c_cparser, "getOne", Falcon::Ext::ConfParser_getOne ).asSymbol()->
+      addParam("key")->addParam("section");
+   self->addClassMethod( c_cparser, "getMultiple", Falcon::Ext::ConfParser_getMultiple ).asSymbol()->
+      addParam("key")->addParam("section");
    self->addClassMethod( c_cparser, "getSections", Falcon::Ext::ConfParser_getSections );
-   self->addClassMethod( c_cparser, "getKeys", Falcon::Ext::ConfParser_getKeys );
-   self->addClassMethod( c_cparser, "getCategoryKeys", Falcon::Ext::ConfParser_getCategoryKeys );
-   self->addClassMethod( c_cparser, "getCategory", Falcon::Ext::ConfParser_getCategory );
-   self->addClassMethod( c_cparser, "removeCategory", Falcon::Ext::ConfParser_removeCategory );
-   self->addClassMethod( c_cparser, "getDictionary", Falcon::Ext::ConfParser_getDictionary );
-   self->addClassMethod( c_cparser, "add", Falcon::Ext::ConfParser_add );
-   self->addClassMethod( c_cparser, "set", Falcon::Ext::ConfParser_set );
-   self->addClassMethod( c_cparser, "remove", Falcon::Ext::ConfParser_remove );
-   self->addClassMethod( c_cparser, "addSection", Falcon::Ext::ConfParser_addSection );
-   self->addClassMethod( c_cparser, "removeSection", Falcon::Ext::ConfParser_removeSection );
+   self->addClassMethod( c_cparser, "getKeys", Falcon::Ext::ConfParser_getKeys ).asSymbol()->
+      addParam("section");
+   self->addClassMethod( c_cparser, "getCategoryKeys", Falcon::Ext::ConfParser_getCategoryKeys ).asSymbol()->
+      addParam("category")->addParam("section");
+   self->addClassMethod( c_cparser, "getCategory", Falcon::Ext::ConfParser_getCategory ).asSymbol()->
+      addParam("category")->addParam("section");
+   self->addClassMethod( c_cparser, "removeCategory", Falcon::Ext::ConfParser_removeCategory ).asSymbol()->
+      addParam("category")->addParam("section");
+   self->addClassMethod( c_cparser, "getDictionary", Falcon::Ext::ConfParser_getDictionary ).asSymbol()->
+      addParam("section");
+   self->addClassMethod( c_cparser, "add", Falcon::Ext::ConfParser_add ).asSymbol()->
+      addParam("key")->addParam("value")->addParam("section");
+   self->addClassMethod( c_cparser, "set", Falcon::Ext::ConfParser_set ).asSymbol()->
+      addParam("key")->addParam("value")->addParam("section");
+   self->addClassMethod( c_cparser, "remove", Falcon::Ext::ConfParser_remove ).asSymbol()->
+      addParam("key")->addParam("section");
+   self->addClassMethod( c_cparser, "addSection", Falcon::Ext::ConfParser_addSection ).asSymbol()->
+      addParam("section");
+   self->addClassMethod( c_cparser, "removeSection", Falcon::Ext::ConfParser_removeSection ).asSymbol()->
+      addParam("section");
    self->addClassMethod( c_cparser, "clearMain", Falcon::Ext::ConfParser_clearMain );
    self->addClassProperty( c_cparser, "errorLine" );
    self->addClassProperty( c_cparser, "error" );
 
+   self->publishService( &s_config_srv );
+
    return self;
 }
 
-/* end of socket.cpp */
+/* end of confparser.cpp */
+
