@@ -31,6 +31,16 @@
 #include <string.h>
 #include "socket_sys.h"
 
+// Sun doesn't provide strerror_r
+#if ( defined (__SUNPRO_CC) && __SUNPRO_CC <= 0x580 )
+static int strerror_r(int errnum, char * buf, unsigned n)
+{
+   ::strncpy(buf, strerror(errnum), n);
+   buf[n-1] = '\0'; // just in case...
+   return 0;
+}
+#endif
+
 namespace Falcon {
 namespace Sys {
 
@@ -420,6 +430,12 @@ bool Socket::bind( Address &addr, bool packet, bool broadcast )
       setsockopt( skt, SOL_SOCKET, SO_BROADCAST, (const char *) &iOpt, sizeof( iOpt ));
    }
 
+   // always useful
+   {
+      int iOpt = 1;
+      setsockopt( skt, SOL_SOCKET, SO_REUSEADDR, (const char *) &iOpt, sizeof( iOpt ));
+   }
+
    int res = ::bind( skt, ai->ai_addr, ai->ai_addrlen );
    d.m_iSystemData = skt;
 
@@ -471,7 +487,7 @@ int32 TCPSocket::recv( byte *buffer, int32 size )
    return read;
 }
 
-int32 TCPSocket::send( byte *buffer, int32 size )
+int32 TCPSocket::send( const byte *buffer, int32 size )
 {
    if ( ! writeAvailable( m_timeout ) ) {
       if ( m_lastError != 0 )
@@ -532,7 +548,7 @@ bool TCPSocket::close()
 bool TCPSocket::connect( Address &where )
 {
    m_lastError = 0;
-   int flags;
+   int flags = 0;
 
    // let's try to connect the addresses in where.
    if ( where.getResolvedCount() == 0 ) {
@@ -661,8 +677,8 @@ bool TCPSocket::isConnected()
 //================================================
 
 ServerSocket::ServerSocket( bool ipv6 ):
-   m_bListening( false ),
-   Socket( 0, ipv6 )
+   Socket( 0, ipv6 ),
+   m_bListening( false )
 {
 }
 
@@ -731,7 +747,6 @@ UDPSocket::UDPSocket( bool ipv6 ):
    Socket( 0, ipv6 )
 {
    // creating an unbound socket
-   int family = ipv6? PF_INET6 : PF_INET;
    int skt = socket( ipv6, SOCK_DGRAM, 0 );
    if ( skt == -1 ) {
       m_lastError = errno;
