@@ -1,22 +1,16 @@
 /*
    FALCON - The Falcon Programming Language.
    FILE: socket_sys_win.cpp
-   $Id: socket_sys_win.cpp,v 1.2 2006/11/07 14:22:43 gian Exp $
 
    MS-Windows system specific interface to sockets.
    -------------------------------------------------------------------
    Author: Giancarlo Niccolai
    Begin: 2006-05-09 15:50
-   Last modified because:
 
    -------------------------------------------------------------------
    (C) Copyright 2004: the FALCON developers (see list in AUTHORS file)
 
    See LICENSE file for licensing details.
-   In order to use this file in its compiled form, this source or
-   part of it you have to read, understand and accept the conditions
-   that are stated in the LICENSE file that comes boundled with this
-   package.
 */
 
 /** \file
@@ -28,7 +22,7 @@
 
 #ifdef __MINGW32__
    #define _inline __inline
-   #include <include/Wspiapi.h> 
+   #include <include/Wspiapi.h>
    #undef _inline
 #else
    #include <Wspiapi.h>
@@ -94,7 +88,7 @@ bool isIPV6( const String &ipv6__ )
    //toCString is guaranteed to stay as long as the string exists.
 	char addrBuff[256];
    ipv6.toCString( addrBuff, 255 );
-   int error = getaddrinfo( addrBuff, 0, &hints, &res );   
+   int error = getaddrinfo( addrBuff, 0, &hints, &res );
 	if ( error == EAI_NONAME )
       return false;
 
@@ -347,7 +341,7 @@ void Socket::terminate()
    d.m_iSystemData = 0;
 }
 
-bool Socket::readAvailable( int32 msec )
+int Socket::readAvailable( int32 msec, const Sys::SystemData *sysData )
 {
    m_lastError = 0;
    int count = s_select( (SOCKET) d.m_iSystemData, msec, 0 );
@@ -358,7 +352,7 @@ bool Socket::readAvailable( int32 msec )
    return count > 0;
 }
 
-bool Socket::writeAvailable( int32 msec )
+int Socket::writeAvailable( int32 msec, const Sys::SystemData *sysData )
 {
    m_lastError = 0;
    int count = s_select( (SOCKET) d.m_iSystemData, msec, 1 );
@@ -415,6 +409,12 @@ bool Socket::bind( Address &addr, bool packet, bool broadcast )
       setsockopt( skt, SOL_SOCKET, SO_BROADCAST, (const char *) &iOpt, sizeof( iOpt ));
    }
 
+   // Always useful
+   {
+      int iOpt = 1;
+      setsockopt( skt, SOL_SOCKET, SO_REUSEADDR, (const char *) &iOpt, sizeof( iOpt ));
+   }
+
    int res = ::bind( skt, ai->ai_addr, ai->ai_addrlen );
    d.m_iSystemData = skt;
 
@@ -466,7 +466,7 @@ int32 TCPSocket::recv( byte *buffer, int32 size )
    return read;
 }
 
-int32 TCPSocket::send( byte *buffer, int32 size )
+int32 TCPSocket::send( const byte *buffer, int32 size )
 {
    if ( ! writeAvailable( m_timeout ) ) {
       if ( m_lastError != 0 )
